@@ -16,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #define BYTES_IN_WORD 4
 
@@ -81,7 +81,8 @@ bfd_reloc_status_type
 
 static void
 MY_swap_std_reloc_in PARAMS ((bfd *abfd, struct reloc_std_external *bytes,
-			      arelent *cache_ptr, asymbol **symbols));
+			      arelent *cache_ptr, asymbol **symbols,
+			      bfd_size_type symcount));
 
 static void
 MY_swap_std_reloc_out PARAMS ((bfd *abfd, arelent *g,
@@ -176,6 +177,9 @@ HOWTO(BFD_RELOC_32_PCREL, 0, 2, 32, true, 0, complain_overflow_signed, 0,
       "PCREL_32", true, 0xffffffff,0xffffffff, false),
 };
 
+
+#define CTOR_TABLE_RELOC_HOWTO(BFD) (MY(howto_table) + 14)
+
 #define RELOC_STD_BITS_NS32K_TYPE_BIG 0x06
 #define RELOC_STD_BITS_NS32K_TYPE_LITTLE 0x60
 #define RELOC_STD_BITS_NS32K_TYPE_SH_BIG 1
@@ -191,7 +195,7 @@ MY(reloc_howto)(abfd, rel, r_index, r_extern, r_pcrel)
 {
   unsigned int r_length;
   int r_ns32k_type;
-/*  BFD_ASSERT(abfd->xvec->header_byteorder_big_p == false); */
+/*  BFD_ASSERT(bfd_header_little_endian (abfd)); */
   *r_index =  ((rel->r_index[2] << 16)
 	       | (rel->r_index[1] << 8)
 	       |  rel->r_index[0] );
@@ -222,7 +226,7 @@ MY(put_reloc)(abfd, r_extern, r_index, value, howto, reloc)
   r_length = howto->size ;	/* Size as a power of two */
   r_pcrel  = (int) howto->pc_relative; /* Relative to PC? */
   r_ns32k_type = (howto - MY(howto_table) )/6;
-/*  BFD_ASSERT (abfd->xvec->header_byteorder_big_p == false); */
+/*  BFD_ASSERT (bfd_header_little_endian (abfd)); */
   reloc->r_index[2] = r_index >> 16;
   reloc->r_index[1] = r_index >> 8;
   reloc->r_index[0] = r_index;
@@ -288,17 +292,16 @@ reloc_howto_type *
 
 
 static void
-MY_swap_std_reloc_in (abfd, bytes, cache_ptr, symbols)
+MY_swap_std_reloc_in (abfd, bytes, cache_ptr, symbols, symcount)
      bfd *abfd;
      struct reloc_std_external *bytes;
      arelent *cache_ptr;
      asymbol **symbols;
+     bfd_size_type symcount;
 {
   int r_index;
   int r_extern;
-  unsigned int r_length;
   int r_pcrel;
-  int r_ns32k_type;
   struct aoutdata  *su = &(abfd->tdata.aout_data->a);
 
   cache_ptr->address = bfd_h_get_32 (abfd, bytes->r_address);
@@ -319,9 +322,6 @@ MY_swap_std_reloc_out (abfd, g, natptr)
   int r_index;
   asymbol *sym = *(g->sym_ptr_ptr);
   int r_extern;
-  unsigned int r_length;
-  int r_pcrel;
-  int r_ns32k_type;
   unsigned int r_addend;
   asection *output_section = sym->section->output_section;
 
@@ -351,8 +351,9 @@ MY_swap_std_reloc_out (abfd, g, natptr)
 	{
 	  /* Fill in symbol */
 	  r_extern = 1;
-#define KEEPIT flags
-	  r_index =  stoi((*(g->sym_ptr_ptr))->KEEPIT);
+#undef KEEPIT
+#define KEEPIT udata.i
+	  r_index =  (*(g->sym_ptr_ptr))->KEEPIT;
 #undef KEEPIT     
 	}
     }
