@@ -16,13 +16,14 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "bfd.h"
 #include "sysdep.h"
 #include "obstack.h"
 #include "libbfd.h"
 #include "bfdlink.h"
+#include "genlink.h"
 #include "coff/h8300.h"
 #include "coff/internal.h"
 #include "libcoff.h"
@@ -76,6 +77,11 @@ static reloc_howto_type howto_table[] =
   HOWTO (R_MOVLB1, 0, 1, 16, false, 0, complain_overflow_bitfield,special, "24/8", false, 0x0000ffff, 0x0000ffff, false),
   HOWTO (R_MOVLB2, 0, 1, 16, false, 0, complain_overflow_bitfield, special, "8/24", false, 0x0000ffff, 0x0000ffff, false),
 
+  /* An indirect reference to a function.  This causes the function's address
+     to be added to the function vector in lo-mem and puts the address of
+     the function vector's entry in the jsr instruction.  */
+  HOWTO (R_MEM_INDIRECT, 0, 0, 8, false, 0, complain_overflow_bitfield, special, "8/indirect", false, 0x000000ff, 0x000000ff, false),
+
 };
 
 
@@ -157,8 +163,11 @@ rtype2howto (internal, dst)
     case R_MOVLB2:
       internal->howto = howto_table + 13;
       break;
+    case R_MEM_INDIRECT:
+      internal->howto = howto_table + 14;
+      break;
     default:
-      fprintf (stderr, "Bad reloc\n");
+      abort ();
       break;
     }
 }
@@ -388,6 +397,7 @@ h8300_reloc16_extra_cases (abfd, link_info, link_order, reloc, data, src_ptr,
 	break;
       }
 
+    case R_MEM_INDIRECT: 	/* Temporary  */
     case R_RELBYTE:
       {
 	unsigned int gap = bfd_coff_reloc16_get_value (reloc, link_info,
@@ -586,6 +596,7 @@ h8300_reloc16_extra_cases (abfd, link_info, link_order, reloc, data, src_ptr,
 #define coff_reloc16_extra_cases h8300_reloc16_extra_cases
 #define coff_reloc16_estimate h8300_reloc16_estimate
 
+#define COFF_LONG_FILENAMES
 #include "coffcode.h"
 
 
@@ -601,8 +612,8 @@ const bfd_target h8300coff_vec =
 {
   "coff-h8300",			/* name */
   bfd_target_coff_flavour,
-  true,				/* data byte order is big */
-  true,				/* header byte order is big */
+  BFD_ENDIAN_BIG,		/* data byte order is big */
+  BFD_ENDIAN_BIG,		/* header byte order is big */
 
   (HAS_RELOC | EXEC_P |		/* object flags */
    HAS_LINENO | HAS_DEBUG |
@@ -611,7 +622,6 @@ const bfd_target h8300coff_vec =
   '_',				/* leading char */
   '/',				/* ar_pad_char */
   15,				/* ar_max_namelen */
-  1,				/* minimum section alignment */
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
   bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* data */
