@@ -15,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifndef TM_I386LYNX_H
 #define TM_I386LYNX_H
@@ -23,7 +23,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "tm-lynx.h"
 
 /* Most definitions from sysv could be used. */
-#include "i386/tm-i386v.h"
+#include "i386/tm-i386.h"
 
 #undef SAVED_PC_AFTER_CALL
 
