@@ -1,5 +1,5 @@
 /* Read dbx symbol tables and convert to internal format, for GDB.
-   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995
+   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996
    Free Software Foundation, Inc.
 
 This file is part of GDB.
@@ -40,7 +40,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include <fcntl.h>
 #endif
 
-#include <obstack.h>
+#include "obstack.h"
 #include <sys/param.h>
 #ifndef	NO_SYS_FILE
 #include <sys/file.h>
@@ -65,10 +65,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include "aout/aout64.h"
 #include "aout/stab_gnu.h"	/* We always use GNU stabs, not native, now */
 
-#if !defined (SEEK_SET)
-#define SEEK_SET 0
-#define SEEK_CUR 1
-#endif
 
 /* We put a pointer to this structure in the read_symtab_private field
    of the psymtab.  */
@@ -246,7 +242,7 @@ static void
 init_bincl_list PARAMS ((int, struct objfile *));
 
 static char *
-dbx_next_symbol_text PARAMS ((void));
+dbx_next_symbol_text PARAMS ((struct objfile *));
 
 static void
 fill_symbuf PARAMS ((bfd *));
@@ -546,6 +542,7 @@ dbx_symfile_read (objfile, section_offsets, mainline)
     ((0 == strncmp (bfd_get_target (objfile->obfd), "elf", 3))
      || (0 == strncmp (bfd_get_target (objfile->obfd), "som", 3))
      || (0 == strncmp (bfd_get_target (objfile->obfd), "coff", 4))
+     || (0 == strncmp (bfd_get_target (objfile->obfd), "pe", 2))
      || (0 == strncmp (bfd_get_target (objfile->obfd), "nlm", 3)));
 
   sym_bfd = objfile->obfd;
@@ -709,6 +706,7 @@ dbx_symfile_init (objfile)
 	  DBX_STRINGTAB (objfile) =
 	    (char *) obstack_alloc (&objfile -> psymbol_obstack,
 				    DBX_STRINGTAB_SIZE (objfile));
+	  OBJSTAT (objfile, sz_strtab += DBX_STRINGTAB_SIZE (objfile));
 	  
 	  /* Now read in the string table in one big gulp.  */
 	  
@@ -828,12 +826,14 @@ fill_symbuf (sym_bfd)
    call this function to get the continuation.  */
 
 static char *
-dbx_next_symbol_text ()
+dbx_next_symbol_text (objfile)
+     struct objfile *objfile;
 {
   if (symbuf_idx == symbuf_end)
     fill_symbuf (symfile_bfd);
   symnum++;
   SWAP_SYMBOL(&symbuf[symbuf_idx], symfile_bfd);
+  OBJSTAT (objfile, n_stabs++);
   return symbuf[symbuf_idx++].n_strx + stringtab_global
 	  + file_string_table_offset;
 }
@@ -1069,6 +1069,7 @@ read_dbx_symtab (section_offsets, objfile, text_addr, text_size)
   int nsl;
   int past_first_source_file = 0;
   CORE_ADDR last_o_file_start = 0;
+  CORE_ADDR last_function_start = 0;
   struct cleanup *back_to;
   bfd *abfd;
 
@@ -1131,6 +1132,7 @@ read_dbx_symtab (section_offsets, objfile, text_addr, text_size)
       if (bufp->n_type == (unsigned char)N_SLINE) continue;
 
       SWAP_SYMBOL (bufp, abfd);
+      OBJSTAT (objfile, n_stabs++);
 
       /* Ok.  There is a lot of code duplicated in the rest of this
          switch statement (for efficiency reasons).  Since I don't
@@ -1210,8 +1212,8 @@ start_psymtab (objfile, section_offsets,
      char *filename;
      CORE_ADDR textlow;
      int ldsymoff;
-     struct partial_symbol *global_syms;
-     struct partial_symbol *static_syms;
+     struct partial_symbol **global_syms;
+     struct partial_symbol **static_syms;
 {
   struct partial_symtab *result =
       start_psymtab_common(objfile, section_offsets,
@@ -1564,6 +1566,7 @@ read_ofile_symtab (pst)
       fill_symbuf (abfd);
       bufp = &symbuf[symbuf_idx++];
       SWAP_SYMBOL (bufp, abfd);
+      OBJSTAT (objfile, n_stabs++);
 
       SET_NAMESTRING ();
 
@@ -1619,6 +1622,7 @@ read_ofile_symtab (pst)
 	fill_symbuf(abfd);
       bufp = &symbuf[symbuf_idx++];
       SWAP_SYMBOL (bufp, abfd);
+      OBJSTAT (objfile, n_stabs++);
 
       type = bufp->n_type;
 
@@ -1672,8 +1676,13 @@ read_ofile_symtab (pst)
   if (last_source_start_addr == 0)
     last_source_start_addr = text_offset;
 
-  pst->symtab = end_symtab (text_offset + text_size, 0, 0, objfile,
-			    SECT_OFF_TEXT);
+  /* In reordered executables last_source_start_addr may not be the
+     lower bound for this symtab, instead use text_offset which comes
+     from pst->textlow which is correct.  */
+  if (last_source_start_addr > text_offset)
+    last_source_start_addr = text_offset;
+
+  pst->symtab = end_symtab (text_offset + text_size, objfile, SECT_OFF_TEXT);
   end_stabs ();
 }
 
@@ -1748,6 +1757,21 @@ process_one_symbol (type, desc, valu, name, section_offsets, objfile)
     {
     case N_FUN:
     case N_FNAME:
+
+      if (! strcmp (name, ""))
+	{
+	  /* This N_FUN marks the end of a function.  This closes off the
+	     current block.  */
+	  within_function = 0;
+	  new = pop_context ();
+
+	  /* Make a block for the local symbols within.  */
+	  finish_block (new->name, &local_symbols, new->old_blocks,
+			function_start_offset, function_start_offset + valu,
+			objfile);
+	  break;
+	}
+
       /* Relocate for dynamic loading */
       valu += ANOFFSET (section_offsets, SECT_OFF_TEXT);
       goto define_a_symbol;
@@ -1899,7 +1923,7 @@ process_one_symbol (type, desc, valu, name, section_offsets, objfile)
 	      patch_subfile_names (current_subfile, name);
 	      break;		/* Ignore repeated SOs */
 	    }
-	  end_symtab (valu, 0, 0, objfile, SECT_OFF_TEXT);
+	  end_symtab (valu, objfile, SECT_OFF_TEXT);
 	  end_stabs ();
 	}
 
@@ -2134,7 +2158,8 @@ process_one_symbol (type, desc, valu, name, section_offsets, objfile)
 		      && SYMBOL_NAME (m) [l] == '\0')
 		    /* last_pc_address was in this function */
 		    valu = SYMBOL_VALUE (m);
-		  else if (m && STREQN (SYMBOL_NAME (m+1), name, l)
+		  else if (m && SYMBOL_NAME (m+1)
+			   && STREQN (SYMBOL_NAME (m+1), name, l)
 			   && SYMBOL_NAME (m+1) [l] == '\0')
 		    /* last_pc_address was in last function */
 		    valu = SYMBOL_VALUE (m+1);
@@ -2271,6 +2296,7 @@ coffstab_build_psymtabs (objfile, section_offsets, mainline,
     error ("ridiculous string table size: %d bytes", stabstrsize);
   DBX_STRINGTAB (objfile) = (char *)
     obstack_alloc (&objfile->psymbol_obstack, stabstrsize+1);
+  OBJSTAT (objfile, sz_strtab += stabstrsize+1);
 
   /* Now read in the string table in one big gulp.  */
 
@@ -2376,6 +2402,7 @@ elfstab_build_psymtabs (objfile, section_offsets, mainline,
     error ("ridiculous string table size: %d bytes", stabstrsize);
   DBX_STRINGTAB (objfile) = (char *)
     obstack_alloc (&objfile->psymbol_obstack, stabstrsize+1);
+  OBJSTAT (objfile, sz_strtab += stabstrsize+1);
 
   /* Now read in the string table in one big gulp.  */
 
@@ -2463,6 +2490,7 @@ stabsect_build_psymtabs (objfile, section_offsets, mainline, stab_name,
     error ("ridiculous string table size: %d bytes", DBX_STRINGTAB_SIZE (objfile));
   DBX_STRINGTAB (objfile) = (char *)
     obstack_alloc (&objfile->psymbol_obstack, DBX_STRINGTAB_SIZE (objfile) + 1);
+  OBJSTAT (objfile, sz_strtab += DBX_STRINGTAB_SIZE (objfile) + 1);
 
   /* Now read in the string table in one big gulp.  */
 
