@@ -1,5 +1,5 @@
 /* Read hp debug symbols and convert to internal format, for GDB.
-   Copyright 1993 Free Software Foundation, Inc.
+   Copyright 1993, 1996 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -32,6 +32,7 @@
 #include "complaints.h"
 #include "gdb-stabs.h"
 #include "gdbtypes.h"
+#include "demangle.h"
 
 /* Private information attached to an objfile which we use to find
    and internalize the HP C debug symbols within that objfile.  */
@@ -160,7 +161,7 @@ void hpread_symfile_finish PARAMS ((struct objfile *));
 
 static struct partial_symtab *hpread_start_psymtab
   PARAMS ((struct objfile *, struct section_offsets *, char *, CORE_ADDR, int,
-	   struct partial_symbol *, struct partial_symbol *));
+	   struct partial_symbol **, struct partial_symbol **));
 
 static struct partial_symtab *hpread_end_psymtab
   PARAMS ((struct partial_symtab *, char **, int, int, CORE_ADDR,
@@ -506,10 +507,10 @@ hpread_build_psymtabs (objfile, section_offsets, mainline)
 	      valu = dn_bufp->dfunc.lowaddr +
 		ANOFFSET (section_offsets, SECT_OFF_TEXT);
 	      SET_NAMESTRING (dn_bufp, &namestring, objfile);
-	      ADD_PSYMBOL_TO_LIST (namestring, strlen (namestring),
+	      add_psymbol_to_list (namestring, strlen (namestring),
 				   VAR_NAMESPACE, LOC_BLOCK,
-				   objfile->static_psymbols, valu,
-				   language_unknown, objfile);
+				   &objfile->static_psymbols, valu,
+				   0, language_unknown, objfile);
 	      within_function = 1;
 	      continue;
 	    case DNTT_TYPE_BEGIN:
@@ -538,7 +539,7 @@ hpread_build_psymtabs (objfile, section_offsets, mainline)
 	      {
 		/* Variables, typedefs an the like.  */
 		enum address_class storage;
-		enum namespace namespace;
+		namespace_enum namespace;
 
 		/* Don't add locals to the partial symbol table.  */
 		if (within_function
@@ -575,19 +576,19 @@ hpread_build_psymtabs (objfile, section_offsets, mainline)
 		  }
 		if (dn_bufp->dsvar.global)
 		  {
-		    ADD_PSYMBOL_TO_LIST (namestring, strlen (namestring),
+		    add_psymbol_to_list (namestring, strlen (namestring),
 					 namespace, storage,
-					 objfile->global_psymbols,
+					 &objfile->global_psymbols,
 					 dn_bufp->dsvar.location,
-					 language_unknown, objfile);
+					 0, language_unknown, objfile);
 		  }
 		else
 		  {
-		    ADD_PSYMBOL_TO_LIST (namestring, strlen (namestring),
+		    add_psymbol_to_list (namestring, strlen (namestring),
 					 namespace, storage,
-					 objfile->static_psymbols,
+					 &objfile->static_psymbols,
 					 dn_bufp->dsvar.location,
-					 language_unknown, objfile);
+					 0, language_unknown, objfile);
 		  }
 		continue;
 	      }
@@ -604,10 +605,10 @@ hpread_build_psymtabs (objfile, section_offsets, mainline)
 					      objfile->global_psymbols.next,
 					      objfile->static_psymbols.next);
 		}
-	      ADD_PSYMBOL_TO_LIST (namestring, strlen (namestring),
+	      add_psymbol_to_list (namestring, strlen (namestring),
 				   VAR_NAMESPACE, LOC_CONST,
-				   objfile->static_psymbols, 0,
-				   language_unknown, objfile);
+				   &objfile->static_psymbols, 0,
+				   0, language_unknown, objfile);
 	      continue;
 	    default:
 	      continue;
@@ -827,8 +828,8 @@ hpread_start_psymtab (objfile, section_offsets,
      char *filename;
      CORE_ADDR textlow;
      int ldsymoff;
-     struct partial_symbol *global_syms;
-     struct partial_symbol *static_syms;
+     struct partial_symbol **global_syms;
+     struct partial_symbol **static_syms;
 {
   struct partial_symtab *result =
   start_psymtab_common (objfile, section_offsets,
@@ -1116,7 +1117,7 @@ hpread_expand_symtab (objfile, sym_offset, sym_size, text_offset, text_size,
 
   current_objfile = NULL;
 
-  return end_symtab (text_offset + text_size, 0, 0, objfile, 0);
+  return end_symtab (text_offset + text_size, objfile, 0);
 }
 
 
@@ -1215,7 +1216,7 @@ hpread_lookup_type (hp_type, objfile)
 	    {
 	      TYPE_VECTOR_LENGTH (objfile) = 100;
 	      TYPE_VECTOR (objfile) = (struct type **)
-		malloc (TYPE_VECTOR_LENGTH (objfile) * sizeof (struct type *));
+		xmalloc (TYPE_VECTOR_LENGTH (objfile) * sizeof (struct type *));
 	    }
 	  while (index >= TYPE_VECTOR_LENGTH (objfile))
 	    TYPE_VECTOR_LENGTH (objfile) *= 2;
@@ -1896,7 +1897,7 @@ hpread_process_one_debug_symbol (dn_bufp, name, section_offsets, objfile,
 	case DNTT_TYPE_MODULE:
 	  /* Ending a module ends the symbol table for that module.  */
 	  valu = text_offset + text_size + offset;
-	  (void) end_symtab (valu, 0, 0, objfile, 0);
+	  (void) end_symtab (valu, objfile, 0);
 	  break;
 
 	case DNTT_TYPE_FUNCTION:
