@@ -1,5 +1,5 @@
 /* GDB routines for manipulating objfiles.
-   Copyright 1992 Free Software Foundation, Inc.
+   Copyright 1992, 1993, 1994, 1995 Free Software Foundation, Inc.
    Contributed by Cygnus Support, using pieces from other GDB modules.
 
 This file is part of GDB.
@@ -16,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* This file contains support routines for creating, manipulating, and
    destroying objfile structures. */
@@ -30,9 +30,10 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "target.h"
 
 #include <sys/types.h>
-#include <sys/stat.h>
+#include "gdb_stat.h"
 #include <fcntl.h>
-#include <obstack.h>
+#include "obstack.h"
+#include "gdb_string.h"
 
 /* Prototypes for local functions */
 
@@ -49,16 +50,13 @@ map_to_address PARAMS ((void));
 
 #endif  /* !defined(NO_MMALLOC) && defined(HAVE_MMAP) */
 
-/* Message to be printed before the error message, when an error occurs.  */
-
-extern char *error_pre_print;
-
 /* Externally visible variables that are owned by this module.
    See declarations in objfile.h for more info. */
 
 struct objfile *object_files;		/* Linked list of all objfiles */
 struct objfile *current_objfile;	/* For symbol file being read in */
 struct objfile *symfile_objfile;	/* Main symbol table loaded from */
+struct objfile *rt_common_objfile;	/* For runtime common symbols */
 
 int mapped_symbol_files;		/* Try to use mapped symbol files */
 
@@ -86,7 +84,7 @@ add_to_objfile_sections (abfd, asect, objfile_p_char)
   section.the_bfd_section = asect;
   section.addr = bfd_section_vma (abfd, asect);
   section.endaddr = section.addr + bfd_section_size (abfd, asect);
-  obstack_grow (&objfile->psymbol_obstack, &section, sizeof(section));
+  obstack_grow (&objfile->psymbol_obstack, (char *) &section, sizeof(section));
   objfile->sections_end = (struct obj_section *) (((unsigned long) objfile->sections_end) + 1);
 }
 
@@ -123,6 +121,7 @@ allocate_objfile (abfd, mapped)
      int mapped;
 {
   struct objfile *objfile = NULL;
+  struct objfile *last_one = NULL;
 
   mapped |= mapped_symbol_files;
 
@@ -160,6 +159,8 @@ allocate_objfile (abfd, mapped)
 	    objfile -> md = md;
 	    objfile -> mmfd = fd;
 	    /* Update pointers to functions to *our* copies */
+	    obstack_chunkfun (&objfile -> psymbol_cache.cache, xmmalloc);
+	    obstack_freefun (&objfile -> psymbol_cache.cache, mfree);
 	    obstack_chunkfun (&objfile -> psymbol_obstack, xmmalloc);
 	    obstack_freefun (&objfile -> psymbol_obstack, mfree);
 	    obstack_chunkfun (&objfile -> symbol_obstack, xmmalloc);
@@ -187,6 +188,9 @@ allocate_objfile (abfd, mapped)
 	    objfile -> mmfd = fd;
 	    objfile -> flags |= OBJF_MAPPED;
 	    mmalloc_setkey (objfile -> md, 0, objfile);
+	    obstack_specify_allocation_with_arg (&objfile -> psymbol_cache.cache,
+						 0, 0, xmmalloc, mfree,
+						 objfile -> md);
 	    obstack_specify_allocation_with_arg (&objfile -> psymbol_obstack,
 						 0, 0, xmmalloc, mfree,
 						 objfile -> md);
@@ -209,7 +213,7 @@ allocate_objfile (abfd, mapped)
 
   if (mapped)
     {
-      warning ("this version of gdb does not support mapped symbol tables.");
+      warning ("mapped symbol tables are not supported on this machine; missing or broken mmap().");
 
       /* Turn off the global flag so we don't try to do mapped symbol tables
 	 any more, which shuts up gdb unless the user specifically gives the
@@ -229,6 +233,8 @@ allocate_objfile (abfd, mapped)
       objfile = (struct objfile *) xmalloc (sizeof (struct objfile));
       memset (objfile, 0, sizeof (struct objfile));
       objfile -> md = NULL;
+      obstack_specify_allocation (&objfile -> psymbol_cache.cache, 0, 0,
+				  xmalloc, free);
       obstack_specify_allocation (&objfile -> psymbol_obstack, 0, 0, xmalloc,
 				  free);
       obstack_specify_allocation (&objfile -> symbol_obstack, 0, 0, xmalloc,
@@ -257,14 +263,42 @@ allocate_objfile (abfd, mapped)
 	     objfile -> name, bfd_errmsg (bfd_get_error ()));
     }
 
-  /* Push this file onto the head of the linked list of other such files. */
-
-  objfile -> next = object_files;
-  object_files = objfile;
+  /* Add this file onto the tail of the linked list of other such files. */
 
+  objfile -> next = NULL;
+  if (object_files == NULL)
+    object_files = objfile;
+  else
+    {
+      for (last_one = object_files;
+	   last_one -> next;
+	   last_one = last_one -> next);
+      last_one -> next = objfile;
+    }
   return (objfile);
 }
 
+/* Put OBJFILE at the front of the list.  */
+
+void
+objfile_to_front (objfile)
+     struct objfile *objfile;
+{
+  struct objfile **objp;
+  for (objp = &object_files; *objp != NULL; objp = &((*objp)->next))
+    {
+      if (*objp == objfile)
+	{
+	  /* Unhook it from where it is.  */
+	  *objp = objfile->next;
+	  /* Put it in the front.  */
+	  objfile->next = object_files;
+	  object_files = objfile;
+	  break;
+	}
+    }
+}
+
 /* Unlink OBJFILE from the list of known objfiles, if it is found in the
    list.
 
@@ -333,7 +367,9 @@ free_objfile (objfile)
   if (objfile -> obfd != NULL)
     {
       char *name = bfd_get_filename (objfile->obfd);
-      bfd_close (objfile -> obfd);
+      if (!bfd_close (objfile -> obfd))
+	warning ("cannot close \"%s\": %s",
+		 name, bfd_errmsg (bfd_get_error ()));
       free (name);
     }
 
@@ -341,6 +377,12 @@ free_objfile (objfile)
 
   unlink_objfile (objfile);
 
+  /* If we are going to free the runtime common objfile, mark it
+     as unallocated.  */
+
+  if (objfile == rt_common_objfile)
+    rt_common_objfile = NULL;
+
   /* Before the symbol table code was redone to make it easier to
      selectively load and remove information particular to a specific
      linkage unit, gdb used to do these things whenever the monolithic
@@ -398,6 +440,7 @@ free_objfile (objfile)
       if (objfile->static_psymbols.list)
 	mfree (objfile->md, objfile->static_psymbols.list);
       /* Free the obstacks for non-reusable objfiles */
+      obstack_free (&objfile -> psymbol_cache.cache, 0);
       obstack_free (&objfile -> psymbol_obstack, 0);
       obstack_free (&objfile -> symbol_obstack, 0);
       obstack_free (&objfile -> type_obstack, 0);
@@ -511,28 +554,24 @@ objfile_relocate (objfile, new_offsets)
 
     ALL_OBJFILE_PSYMTABS (objfile, p)
       {
-	/* FIXME: specific to symbol readers which use gdb-stabs.h.
-	   We can only get away with it since objfile_relocate is only
-	   used on XCOFF, which lacks psymtabs, and for gdb-stabs.h
-	   targets.  */
 	p->textlow += ANOFFSET (delta, SECT_OFF_TEXT);
 	p->texthigh += ANOFFSET (delta, SECT_OFF_TEXT);
       }
   }
 
   {
-    struct partial_symbol *psym;
+    struct partial_symbol **psym;
 
     for (psym = objfile->global_psymbols.list;
 	 psym < objfile->global_psymbols.next;
 	 psym++)
-      if (SYMBOL_SECTION (psym) >= 0)
-	SYMBOL_VALUE_ADDRESS (psym) += ANOFFSET (delta, SYMBOL_SECTION (psym));
+      if (SYMBOL_SECTION (*psym) >= 0)
+	SYMBOL_VALUE_ADDRESS (*psym) += ANOFFSET (delta, SYMBOL_SECTION (*psym));
     for (psym = objfile->static_psymbols.list;
 	 psym < objfile->static_psymbols.next;
 	 psym++)
-      if (SYMBOL_SECTION (psym) >= 0)
-	SYMBOL_VALUE_ADDRESS (psym) += ANOFFSET (delta, SYMBOL_SECTION (psym));
+      if (SYMBOL_SECTION (*psym) >= 0)
+	SYMBOL_VALUE_ADDRESS (*psym) += ANOFFSET (delta, SYMBOL_SECTION (*psym));
   }
 
   {
@@ -541,6 +580,9 @@ objfile_relocate (objfile, new_offsets)
       if (SYMBOL_SECTION (msym) >= 0)
 	SYMBOL_VALUE_ADDRESS (msym) += ANOFFSET (delta, SYMBOL_SECTION (msym));
   }
+  /* Relocating different sections by different amounts may cause the symbols
+     to be out of order.  */
+  msymbols_sort (objfile);
 
   {
     int i;
@@ -552,10 +594,10 @@ objfile_relocate (objfile, new_offsets)
     struct obj_section *s;
     bfd *abfd;
 
-    abfd = symfile_objfile->obfd;
+    abfd = objfile->obfd;
 
-    for (s = symfile_objfile->sections;
-	 s < symfile_objfile->sections_end; ++s)
+    for (s = objfile->sections;
+	 s < objfile->sections_end; ++s)
       {
 	flagword flags;
 
@@ -821,6 +863,7 @@ map_to_address ()
 
 #else
 
+  warning ("need to recompile gdb with MMAP_BASE_ADDRESS and MMAP_INCREMENT defined");
   return (0);
 
 #endif
