@@ -1,5 +1,5 @@
 /* Read os9/os9k symbol tables and convert to internal format, for GDB.
-   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994
+   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1996
    Free Software Foundation, Inc.
 
 This file is part of GDB.
@@ -41,7 +41,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include <fcntl.h>
 #endif
 
-#include <obstack.h>
+#include "obstack.h"
 #include <sys/param.h>
 #ifndef	NO_SYS_FILE
 #include <sys/file.h>
@@ -64,11 +64,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include "os9k.h"
 #include "stabsread.h"
 
-#if !defined (SEEK_SET)
-#define SEEK_SET 0
-#define SEEK_CUR 1
-#endif
-
 /* Each partial symbol table entry contains a pointer to private data for the
    read_symtab() function to use when expanding a partial symbol table entry
    to a full symbol table entry.
@@ -164,8 +159,8 @@ os9k_process_one_symbol PARAMS ((int, int, CORE_ADDR, char *,
 
 static struct partial_symtab *
 os9k_start_psymtab PARAMS ((struct objfile *, struct section_offsets *, char *,
-                       CORE_ADDR, int, int, struct partial_symbol *,
-                       struct partial_symbol *));
+                       CORE_ADDR, int, int, struct partial_symbol **,
+                       struct partial_symbol **));
 
 static struct partial_symtab *
 os9k_end_psymtab PARAMS ((struct partial_symtab *, char **, int, int, CORE_ADDR,
@@ -742,35 +737,35 @@ read_os9k_psymtab (section_offsets, objfile, text_addr, text_size)
 	      }
 
 	    case 'v':
-	      ADD_PSYMBOL_ADDR_TO_LIST (namestring, p - namestring,
-					VAR_NAMESPACE, LOC_STATIC,
-					objfile->static_psymbols,
-					CUR_SYMBOL_VALUE,
-					psymtab_language, objfile);
+	      add_psymbol_to_list (namestring, p - namestring,
+				   VAR_NAMESPACE, LOC_STATIC,
+				   &objfile->static_psymbols,
+				   0, CUR_SYMBOL_VALUE,
+				   psymtab_language, objfile);
 	      continue;
 	    case 'V':
-	      ADD_PSYMBOL_ADDR_TO_LIST (namestring, p - namestring,
-					VAR_NAMESPACE, LOC_STATIC,
-					objfile->global_psymbols,
-					CUR_SYMBOL_VALUE,
-					psymtab_language, objfile);
+	      add_psymbol_to_list (namestring, p - namestring,
+				   VAR_NAMESPACE, LOC_STATIC,
+				   &objfile->global_psymbols,
+				   0, CUR_SYMBOL_VALUE,
+				   psymtab_language, objfile);
 	      continue;
 
 	    case 'T':
 	      if (p != namestring)	/* a name is there, not just :T... */
 		{
-		  ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+		  add_psymbol_to_list (namestring, p - namestring,
 				       STRUCT_NAMESPACE, LOC_TYPEDEF,
-				       objfile->static_psymbols,
-				       CUR_SYMBOL_VALUE,
+				       &objfile->static_psymbols,
+				       CUR_SYMBOL_VALUE, 0,
 				       psymtab_language, objfile);
 		  if (p[2] == 't')
 		    {
 		      /* Also a typedef with the same name.  */
-		      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+		      add_psymbol_to_list (namestring, p - namestring,
 					   VAR_NAMESPACE, LOC_TYPEDEF,
-					   objfile->static_psymbols,
-					   CUR_SYMBOL_VALUE, psymtab_language,
+					   &objfile->static_psymbols,
+					   CUR_SYMBOL_VALUE, 0, psymtab_language,
 					   objfile);
 		      p += 1;
 		    }
@@ -782,10 +777,10 @@ read_os9k_psymtab (section_offsets, objfile, text_addr, text_size)
 		  else if (psymtab_language == language_cplus)
 		   {
 		      /* Also a typedef with the same name.  */
-		      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+		      add_psymbol_to_list (namestring, p - namestring,
 					   VAR_NAMESPACE, LOC_TYPEDEF,
-					   objfile->static_psymbols,
-					   CUR_SYMBOL_VALUE, psymtab_language,
+					   &objfile->static_psymbols,
+					   CUR_SYMBOL_VALUE, 0, psymtab_language,
 					   objfile);
 		   }
 		}
@@ -793,10 +788,10 @@ read_os9k_psymtab (section_offsets, objfile, text_addr, text_size)
 	    case 't':
 	      if (p != namestring)	/* a name is there, not just :T... */
 		{
-		  ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+		  add_psymbol_to_list (namestring, p - namestring,
 				       VAR_NAMESPACE, LOC_TYPEDEF,
-				       objfile->static_psymbols,
-				       CUR_SYMBOL_VALUE,
+				       &objfile->static_psymbols,
+				       CUR_SYMBOL_VALUE, 0,
 				       psymtab_language, objfile);
 		}
 	    check_enum:
@@ -837,7 +832,7 @@ read_os9k_psymtab (section_offsets, objfile, text_addr, text_size)
 		      /* Check for and handle cretinous dbx symbol name
 			 continuation! 
 		      if (*p == '\\')
-			p = next_symbol_text ();
+			p = next_symbol_text (objfile);
 		      */
 
 		      /* Point to the character after the name
@@ -846,10 +841,10 @@ read_os9k_psymtab (section_offsets, objfile, text_addr, text_size)
 			;
 		      /* Note that the value doesn't matter for
 			 enum constants in psymtabs, just in symtabs.  */
-		      ADD_PSYMBOL_TO_LIST (p, q - p,
+		      add_psymbol_to_list (p, q - p,
 					   VAR_NAMESPACE, LOC_CONST,
-					   objfile->static_psymbols, 0,
-					   psymtab_language, objfile);
+					   &objfile->static_psymbols, 0,
+					   0, psymtab_language, objfile);
 		      /* Point past the name.  */
 		      p = q;
 		      /* Skip over the value.  */
@@ -863,10 +858,10 @@ read_os9k_psymtab (section_offsets, objfile, text_addr, text_size)
 	      continue;
 	    case 'c':
 	      /* Constant, e.g. from "const" in Pascal.  */
-	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+	      add_psymbol_to_list (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_CONST,
-				   objfile->static_psymbols, CUR_SYMBOL_VALUE,
-				   psymtab_language, objfile);
+				   &objfile->static_psymbols, CUR_SYMBOL_VALUE,
+				   0, psymtab_language, objfile);
 	      continue;
 
 	    case 'f':
@@ -874,10 +869,10 @@ read_os9k_psymtab (section_offsets, objfile, text_addr, text_size)
               if (pst && pst->textlow == 0)
                 pst->textlow = CUR_SYMBOL_VALUE;
 
-	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+	      add_psymbol_to_list (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_BLOCK,
-				   objfile->static_psymbols, CUR_SYMBOL_VALUE,
-				   psymtab_language, objfile);
+				   &objfile->static_psymbols, CUR_SYMBOL_VALUE,
+				   0, psymtab_language, objfile);
 	      continue;
 
 	    case 'F':
@@ -885,10 +880,10 @@ read_os9k_psymtab (section_offsets, objfile, text_addr, text_size)
               if (pst && pst->textlow == 0)
                 pst->textlow = CUR_SYMBOL_VALUE;
 
-	      ADD_PSYMBOL_TO_LIST (namestring, p - namestring,
+	      add_psymbol_to_list (namestring, p - namestring,
 				   VAR_NAMESPACE, LOC_BLOCK,
-				   objfile->global_psymbols, CUR_SYMBOL_VALUE,
-				   psymtab_language, objfile);
+				   &objfile->global_psymbols, CUR_SYMBOL_VALUE,
+				   0, psymtab_language, objfile);
 	      continue;
 
 	    case 'p':
@@ -977,8 +972,8 @@ os9k_start_psymtab (objfile, section_offsets,
      CORE_ADDR textlow;
      int ldsymoff;
      int ldsymcnt;
-     struct partial_symbol *global_syms;
-     struct partial_symbol *static_syms;
+     struct partial_symbol **global_syms;
+     struct partial_symbol **static_syms;
 {
   struct partial_symtab *result =
       start_psymtab_common(objfile, section_offsets,
@@ -1414,8 +1409,7 @@ os9k_read_ofile_symtab (pst)
      which comes from pst->textlow is correct. */
   if (last_source_start_addr == 0)
     last_source_start_addr = text_offset;
-  pst->symtab = end_symtab (text_offset + text_size, 0, 0, objfile,
-			    SECT_OFF_TEXT);
+  pst->symtab = end_symtab (text_offset + text_size, objfile, SECT_OFF_TEXT);
   end_stabs ();
 }
 
@@ -1534,9 +1528,10 @@ os9k_process_one_symbol (type, desc, valu, name, section_offsets, objfile)
     case N_SYM_SLINE:
       /* This type of "symbol" really just records
 	 one line-number -- core-address correspondence.
-	 Enter it in the line list for this symbol table.  */
+	 Enter it in the line list for this symbol table. */
       /* Relocate for dynamic loading and for ELF acc fn-relative syms.  */
       valu += ANOFFSET (section_offsets, SECT_OFF_TEXT); 
+      /* FIXME: loses if sizeof (char *) > sizeof (int) */
       record_line (current_subfile, (int)name, valu);
       break;
 
@@ -1571,7 +1566,7 @@ os9k_process_one_symbol (type, desc, valu, name, section_offsets, objfile)
 	      *p = '\0';
 	      if (symfile_depth++ == 0) {
 		if (last_source_file) {
-          	  end_symtab (valu, 0, 0, objfile, SECT_OFF_TEXT);
+          	  end_symtab (valu, objfile, SECT_OFF_TEXT);
 		  end_stabs ();
 		}
 		start_stabs ();
