@@ -17,7 +17,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #if !defined (PARSER_DEFS_H)
 #define PARSER_DEFS_H 1
@@ -95,7 +95,7 @@ extern void write_exp_elt_sym PARAMS ((struct symbol *));
 
 extern void write_exp_elt_longcst PARAMS ((LONGEST));
 
-extern void write_exp_elt_dblcst PARAMS ((double));
+extern void write_exp_elt_dblcst PARAMS ((DOUBLEST));
 
 extern void write_exp_elt_type PARAMS ((struct type *));
 
@@ -110,6 +110,8 @@ extern void write_exp_elt_block PARAMS ((struct block *));
 extern void write_exp_msymbol PARAMS ((struct minimal_symbol *,
 				       struct type *, struct type *));
 
+extern void write_dollar_variable PARAMS ((struct stoken str));
+
 extern void
 start_arglist PARAMS ((void));
 
@@ -169,7 +171,7 @@ enum precedence
 { PREC_NULL, PREC_COMMA, PREC_ABOVE_COMMA, PREC_ASSIGN, PREC_LOGICAL_OR,
   PREC_LOGICAL_AND, PREC_BITWISE_IOR, PREC_BITWISE_AND, PREC_BITWISE_XOR,
   PREC_EQUAL, PREC_ORDER, PREC_SHIFT, PREC_ADD, PREC_MUL, PREC_REPEAT,
-  PREC_HYPER, PREC_PREFIX, PREC_SUFFIX };
+  PREC_HYPER, PREC_PREFIX, PREC_SUFFIX, PREC_BUILTIN_FUNCTION };
 
 /* Table mapping opcodes into strings for printing operators
    and precedences of the operators.  */
