@@ -1,5 +1,5 @@
 /* Do various things to symbol tables (other than lookup), for GDB.
-   Copyright 1986, 1987, 1989, 1991, 1992, 1993, 1994
+   Copyright 1986, 1987, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Free Software Foundation, Inc.
 
 This file is part of GDB.
@@ -16,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "defs.h"
 #include "symtab.h"
@@ -28,8 +28,9 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "command.h"
 #include "obstack.h"
 #include "language.h"
+#include "bcache.h"
 
-#include <string.h>
+#include "gdb_string.h"
 
 #ifndef DEV_TTY
 #define DEV_TTY "/dev/tty"
@@ -63,7 +64,7 @@ static int
 block_depth PARAMS ((struct block *));
 
 static void
-print_partial_symbol PARAMS ((struct partial_symbol *, int, char *, GDB_FILE *));
+print_partial_symbols PARAMS ((struct partial_symbol **, int, char *, GDB_FILE *));
 
 struct print_symbol_args {
   struct symbol *symbol;
@@ -152,6 +153,59 @@ free_symtab (s)
 
 #if MAINTENANCE_CMDS
 
+void
+print_symbol_bcache_statistics ()
+{
+  struct objfile *objfile;
+
+  immediate_quit++;
+  ALL_OBJFILES (objfile)
+    {
+      printf_filtered ("Byte cache statistics for '%s':\n", objfile -> name);
+      print_bcache_statistics (&objfile -> psymbol_cache, "partial symbol cache");
+    }
+  immediate_quit--;
+}
+
+void
+print_objfile_statistics ()
+{
+  struct objfile *objfile;
+
+  immediate_quit++;
+  ALL_OBJFILES (objfile)
+    {
+      printf_filtered ("Statistics for '%s':\n", objfile -> name);
+      if (OBJSTAT (objfile, n_stabs) > 0)
+	printf_filtered ("  Number of \"stab\" symbols read: %d\n",
+			 OBJSTAT (objfile, n_stabs));
+      if (OBJSTAT (objfile, n_minsyms) > 0)
+	printf_filtered ("  Number of \"minimal symbols read: %d\n",
+			 OBJSTAT (objfile, n_minsyms));
+      if (OBJSTAT (objfile, n_psyms) > 0)
+	printf_filtered ("  Number of \"partial symbols read: %d\n",
+			 OBJSTAT (objfile, n_psyms));
+      if (OBJSTAT (objfile, n_syms) > 0)
+	printf_filtered ("  Number of \"full symbols read: %d\n",
+			 OBJSTAT (objfile, n_syms));
+      if (OBJSTAT (objfile, n_types) > 0)
+	printf_filtered ("  Number of \"types defined: %d\n",
+			 OBJSTAT (objfile, n_types));
+      if (OBJSTAT (objfile, sz_strtab) > 0)
+	printf_filtered ("  Space used by a.out string tables: %d\n",
+			 OBJSTAT (objfile, sz_strtab));
+      printf_filtered ("  Total memory used for psymbol obstack: %d\n",
+		       obstack_memory_used (&objfile -> psymbol_obstack));
+      printf_filtered ("  Total memory used for psymbol cache: %d\n",
+		       obstack_memory_used (&objfile -> psymbol_cache.cache));
+      printf_filtered ("  Total memory used for symbol obstack: %d\n",
+		       obstack_memory_used (&objfile -> symbol_obstack));
+      printf_filtered ("  Total memory used for type obstack: %d\n",
+		       obstack_memory_used (&objfile -> type_obstack));
+    }
+  immediate_quit--;
+}
+
 static void 
 dump_objfile (objfile)
      struct objfile *objfile;
@@ -336,13 +390,13 @@ dump_psymtab (objfile, psymtab, outfile)
     }
   if (psymtab -> n_global_syms > 0)
     {
-      print_partial_symbol (objfile -> global_psymbols.list
+      print_partial_symbols (objfile -> global_psymbols.list
 			    + psymtab -> globals_offset,
 			    psymtab -> n_global_syms, "Global", outfile);
     }
   if (psymtab -> n_static_syms > 0)
     {
-      print_partial_symbol (objfile -> static_psymbols.list
+      print_partial_symbols (objfile -> static_psymbols.list
 			    + psymtab -> statics_offset,
 			    psymtab -> n_static_syms, "Static", outfile);
     }
@@ -424,7 +478,7 @@ dump_symtab (objfile, symtab, outfile)
 	  s.depth = depth + 1;
 	  s.outfile = outfile;
 	  catch_errors (print_symbol, &s, "Error printing symbol:\n",
-			RETURN_MASK_ERROR);
+			RETURN_MASK_ALL);
 	}
     }
   fprintf_filtered (outfile, "\n");
@@ -546,13 +600,14 @@ print_symbol (args)
 	  break;
 
 	case LOC_CONST_BYTES:
-	  fprintf_filtered (outfile, "const %u hex bytes:",
-		   TYPE_LENGTH (SYMBOL_TYPE (symbol)));
 	  {
 	    unsigned i;
-	    for (i = 0; i < TYPE_LENGTH (SYMBOL_TYPE (symbol)); i++)
+	    struct type *type = check_typedef (SYMBOL_TYPE (symbol));
+	    fprintf_filtered (outfile, "const %u hex bytes:",
+			      TYPE_LENGTH (type));
+	    for (i = 0; i < TYPE_LENGTH (type); i++)
 	      fprintf_filtered (outfile, " %02x",
-			 (unsigned)SYMBOL_VALUE_BYTES (symbol) [i]);
+				(unsigned)SYMBOL_VALUE_BYTES (symbol) [i]);
 	    fprintf_filtered (outfile, ",");
 	  }
 	  break;
@@ -622,6 +677,10 @@ print_symbol (args)
 	  fprintf_filtered (outfile, ",");
 	  break;
 
+	case LOC_UNRESOLVED:
+	  fprintf_filtered (outfile, "unresolved");
+	  break;
+
 	case LOC_OPTIMIZED_OUT:
 	  fprintf_filtered (outfile, "optimized out");
 	  break;
@@ -688,23 +747,22 @@ maintenance_print_psymbols (args, from_tty)
 }
 
 static void
-print_partial_symbol (p, count, what, outfile)
-     struct partial_symbol *p;
+print_partial_symbols (p, count, what, outfile)
+     struct partial_symbol **p;
      int count;
      char *what;
      GDB_FILE *outfile;
 {
-
   fprintf_filtered (outfile, "  %s partial symbols:\n", what);
   while (count-- > 0)
     {
-      fprintf_filtered (outfile, "    `%s'", SYMBOL_NAME(p));
-      if (SYMBOL_DEMANGLED_NAME (p) != NULL)
+      fprintf_filtered (outfile, "    `%s'", SYMBOL_NAME(*p));
+      if (SYMBOL_DEMANGLED_NAME (*p) != NULL)
 	{
-	  fprintf_filtered (outfile, "  `%s'", SYMBOL_DEMANGLED_NAME (p));
+	  fprintf_filtered (outfile, "  `%s'", SYMBOL_DEMANGLED_NAME (*p));
 	}
       fputs_filtered (", ", outfile);
-      switch (SYMBOL_NAMESPACE (p))
+      switch (SYMBOL_NAMESPACE (*p))
 	{
 	case UNDEF_NAMESPACE:
 	  fputs_filtered ("undefined namespace, ", outfile);
@@ -722,7 +780,7 @@ print_partial_symbol (p, count, what, outfile)
 	  fputs_filtered ("<invalid namespace>, ", outfile);
 	  break;
 	}
-      switch (SYMBOL_CLASS (p))
+      switch (SYMBOL_CLASS (*p))
 	{
 	case LOC_UNDEF:
 	  fputs_filtered ("undefined", outfile);
@@ -766,6 +824,9 @@ print_partial_symbol (p, count, what, outfile)
 	case LOC_LOCAL_ARG:
 	  fputs_filtered ("shuffled arg", outfile);
 	  break;
+	case LOC_UNRESOLVED:
+	  fputs_filtered ("unresolved", outfile);
+	  break;
 	case LOC_OPTIMIZED_OUT:
 	  fputs_filtered ("optimized out", outfile);
 	  break;
@@ -777,7 +838,7 @@ print_partial_symbol (p, count, what, outfile)
       /* FIXME-32x64: Need to use SYMBOL_VALUE_ADDRESS, etc.; this
 	 could be 32 bits when some of the other fields in the union
 	 are 64.  */
-      fprintf_filtered (outfile, "0x%lx\n", SYMBOL_VALUE (p));
+      fprintf_filtered (outfile, "0x%lx\n", SYMBOL_VALUE (*p));
       p++;
     }
 }
@@ -856,7 +917,7 @@ maintenance_check_symtabs (ignore, from_tty)
      int from_tty;
 {
   register struct symbol *sym;
-  register struct partial_symbol *psym;
+  register struct partial_symbol **psym;
   register struct symtab *s = NULL;
   register struct partial_symtab *ps;
   struct blockvector *bv;
@@ -875,12 +936,12 @@ maintenance_check_symtabs (ignore, from_tty)
       length = ps->n_static_syms;
       while (length--)
 	{
-	  sym = lookup_block_symbol (b, SYMBOL_NAME (psym),
-				     SYMBOL_NAMESPACE (psym));
+	  sym = lookup_block_symbol (b, SYMBOL_NAME (*psym),
+				     SYMBOL_NAMESPACE (*psym));
 	  if (!sym)
 	    {
 	      printf_filtered ("Static symbol `");
-	      puts_filtered (SYMBOL_NAME (psym));
+	      puts_filtered (SYMBOL_NAME (*psym));
 	      printf_filtered ("' only found in ");
 	      puts_filtered (ps->filename);
 	      printf_filtered (" psymtab\n");
@@ -892,12 +953,12 @@ maintenance_check_symtabs (ignore, from_tty)
       length = ps->n_global_syms;
       while (length--)
 	{
-	  sym = lookup_block_symbol (b, SYMBOL_NAME (psym),
-				     SYMBOL_NAMESPACE (psym));
+	  sym = lookup_block_symbol (b, SYMBOL_NAME (*psym),
+				     SYMBOL_NAMESPACE (*psym));
 	  if (!sym)
 	    {
 	      printf_filtered ("Global symbol `");
-	      puts_filtered (SYMBOL_NAME (psym));
+	      puts_filtered (SYMBOL_NAME (*psym));
 	      printf_filtered ("' only found in ");
 	      puts_filtered (ps->filename);
 	      printf_filtered (" psymtab\n");
@@ -953,7 +1014,7 @@ block_depth (block)
 
 
 /* Increase the space allocated for LISTP, which is probably
-   global_psymbol_list or static_psymbol_list. This space will eventually
+   global_psymbols or static_psymbols. This space will eventually
    be freed in free_objfile().  */
 
 void
@@ -965,15 +1026,15 @@ extend_psymbol_list (listp, objfile)
   if (listp->size == 0)
     {
       new_size = 255;
-      listp->list = (struct partial_symbol *)
-	xmmalloc (objfile -> md, new_size * sizeof (struct partial_symbol));
+      listp->list = (struct partial_symbol **)
+	xmmalloc (objfile -> md, new_size * sizeof (struct partial_symbol *));
     }
   else
     {
       new_size = listp->size * 2;
-      listp->list = (struct partial_symbol *)
+      listp->list = (struct partial_symbol **)
 	xmrealloc (objfile -> md, (char *) listp->list,
-		   new_size * sizeof (struct partial_symbol));
+		   new_size * sizeof (struct partial_symbol *));
     }
   /* Next assumes we only went one over.  Should be good if
      program works correctly */
