@@ -32,6 +32,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include <errno.h>
 #include "gdb_string.h"
 
+/* Default to coercing float to double in function calls only when there is
+   no prototype.  Otherwise on targets where the debug information is incorrect
+   for either the prototype or non-prototype case, we can force it by defining
+   COERCE_FLOAT_TO_DOUBLE in the target configuration file. */
+
+#ifndef COERCE_FLOAT_TO_DOUBLE
+#define COERCE_FLOAT_TO_DOUBLE (param_type == NULL)
+#endif
+
 /* Local functions.  */
 
 static int typecmp PARAMS ((int staticp, struct type *t1[], value_ptr t2[]));
@@ -55,48 +64,59 @@ static value_ptr cast_into_complex PARAMS ((struct type *, value_ptr));
 
 #define VALUE_SUBSTRING_START(VAL) VALUE_FRAME(VAL)
 
+/* Flag for whether we want to abandon failed expression evals by default.  */
+
+#if 0
+static int auto_abandon = 0;
+#endif
+
 
-/* Allocate NBYTES of space in the inferior using the inferior's malloc
-   and return a value that is a pointer to the allocated space. */
+/* Find the address of function name NAME in the inferior.  */
 
-static CORE_ADDR
-allocate_space_in_inferior (len)
-     int len;
+value_ptr
+find_function_in_inferior (name)
+     char *name;
 {
-  register value_ptr val;
   register struct symbol *sym;
-  struct minimal_symbol *msymbol;
-  struct type *type;
-  value_ptr blocklen;
-  LONGEST maddr;
-
-  /* Find the address of malloc in the inferior.  */
-
-  sym = lookup_symbol ("malloc", 0, VAR_NAMESPACE, 0, NULL);
+  sym = lookup_symbol (name, 0, VAR_NAMESPACE, 0, NULL);
   if (sym != NULL)
     {
       if (SYMBOL_CLASS (sym) != LOC_BLOCK)
 	{
-	  error ("\"malloc\" exists in this program but is not a function.");
+	  error ("\"%s\" exists in this program but is not a function.",
+		 name);
 	}
-      val = value_of_variable (sym, NULL);
+      return value_of_variable (sym, NULL);
     }
   else
     {
-      msymbol = lookup_minimal_symbol ("malloc", NULL, NULL);
+      struct minimal_symbol *msymbol = lookup_minimal_symbol(name, NULL, NULL);
       if (msymbol != NULL)
 	{
+	  struct type *type;
+	  LONGEST maddr;
 	  type = lookup_pointer_type (builtin_type_char);
 	  type = lookup_function_type (type);
 	  type = lookup_pointer_type (type);
 	  maddr = (LONGEST) SYMBOL_VALUE_ADDRESS (msymbol);
-	  val = value_from_longest (type, maddr);
+	  return value_from_longest (type, maddr);
 	}
       else
 	{
-	  error ("evaluation of this expression requires the program to have a function \"malloc\".");
+	  error ("evaluation of this expression requires the program to have a function \"%s\".", name);
 	}
     }
+}
+
+/* Allocate NBYTES of space in the inferior using the inferior's malloc
+   and return a value that is a pointer to the allocated space. */
+
+value_ptr
+value_allocate_space_in_inferior (len)
+     int len;
+{
+  value_ptr blocklen;
+  register value_ptr val = find_function_in_inferior ("malloc");
 
   blocklen = value_from_longest (builtin_type_int, (LONGEST) len);
   val = call_function_by_hand (val, 1, &blocklen);
@@ -104,7 +124,14 @@ allocate_space_in_inferior (len)
     {
       error ("No memory available to program.");
     }
-  return (value_as_long (val));
+  return val;
+}
+
+static CORE_ADDR
+allocate_space_in_inferior (len)
+     int len;
+{
+  return value_as_long (value_allocate_space_in_inferior (len));
 }
 
 /* Cast value ARG2 to type TYPE and return as a value.
@@ -117,56 +144,65 @@ value_cast (type, arg2)
      struct type *type;
      register value_ptr arg2;
 {
-  register enum type_code code1 = TYPE_CODE (type);
+  register enum type_code code1;
   register enum type_code code2;
   register int scalar;
+  struct type *type2;
 
   if (VALUE_TYPE (arg2) == type)
     return arg2;
 
+  CHECK_TYPEDEF (type);
+  code1 = TYPE_CODE (type);
   COERCE_REF(arg2);
+  type2 = check_typedef (VALUE_TYPE (arg2));
 
   /* A cast to an undetermined-length array_type, such as (TYPE [])OBJECT,
      is treated like a cast to (TYPE [N])OBJECT,
      where N is sizeof(OBJECT)/sizeof(TYPE). */
-  if (code1 == TYPE_CODE_ARRAY
-      && TYPE_LENGTH (TYPE_TARGET_TYPE (type)) > 0
-      && TYPE_ARRAY_UPPER_BOUND_TYPE (type) == BOUND_CANNOT_BE_DETERMINED)
+  if (code1 == TYPE_CODE_ARRAY)
     {
       struct type *element_type = TYPE_TARGET_TYPE (type);
-      struct type *range_type = TYPE_INDEX_TYPE (type);
-      int low_bound = TYPE_LOW_BOUND (range_type);
-      int val_length = TYPE_LENGTH (VALUE_TYPE (arg2));
-      int new_length = val_length / TYPE_LENGTH (element_type);
-      if (val_length % TYPE_LENGTH (element_type) != 0)
-	warning("array element type size does not divide object size in cast");
-      /* FIXME-type-allocation: need a way to free this type when we are
-	 done with it.  */
-      range_type = create_range_type ((struct type *) NULL,
-				      TYPE_TARGET_TYPE (range_type),
-				      low_bound, new_length + low_bound - 1);
-      VALUE_TYPE (arg2) = create_array_type ((struct type *) NULL,
-					     element_type, range_type);
-      return arg2;
+      unsigned element_length = TYPE_LENGTH (check_typedef (element_type));
+      if (element_length > 0
+	  && TYPE_ARRAY_UPPER_BOUND_TYPE (type) == BOUND_CANNOT_BE_DETERMINED)
+	{
+	  struct type *range_type = TYPE_INDEX_TYPE (type);
+	  int val_length = TYPE_LENGTH (type2);
+	  LONGEST low_bound, high_bound, new_length;
+	  if (get_discrete_bounds (range_type, &low_bound, &high_bound) < 0)
+	    low_bound = 0, high_bound = 0;
+	  new_length = val_length / element_length;
+	  if (val_length % element_length != 0)
+       warning("array element type size does not divide object size in cast");
+	  /* FIXME-type-allocation: need a way to free this type when we are
+	     done with it.  */
+	  range_type = create_range_type ((struct type *) NULL,
+					  TYPE_TARGET_TYPE (range_type),
+					  low_bound,
+					  new_length + low_bound - 1);
+	  VALUE_TYPE (arg2) = create_array_type ((struct type *) NULL,
+						 element_type, range_type);
+	  return arg2;
+	}
     }
 
   if (current_language->c_style_arrays
-      && (VALUE_REPEATED (arg2)
-	  || TYPE_CODE (VALUE_TYPE (arg2)) == TYPE_CODE_ARRAY))
+      && TYPE_CODE (type2) == TYPE_CODE_ARRAY)
     arg2 = value_coerce_array (arg2);
 
-  if (TYPE_CODE (VALUE_TYPE (arg2)) == TYPE_CODE_FUNC)
+  if (TYPE_CODE (type2) == TYPE_CODE_FUNC)
     arg2 = value_coerce_function (arg2);
 
-  COERCE_VARYING_ARRAY (arg2);
+  type2 = check_typedef (VALUE_TYPE (arg2));
+  COERCE_VARYING_ARRAY (arg2, type2);
+  code2 = TYPE_CODE (type2);
 
-  code2 = TYPE_CODE (VALUE_TYPE (arg2));
-
-  if (code1 == TYPE_CODE_COMPLEX) 
-    return cast_into_complex (type, arg2); 
-  if (code1 == TYPE_CODE_BOOL) 
+  if (code1 == TYPE_CODE_COMPLEX)
+    return cast_into_complex (type, arg2);
+  if (code1 == TYPE_CODE_BOOL || code1 == TYPE_CODE_CHAR)
     code1 = TYPE_CODE_INT; 
-  if (code2 == TYPE_CODE_BOOL) 
+  if (code2 == TYPE_CODE_BOOL || code2 == TYPE_CODE_CHAR)
     code2 = TYPE_CODE_INT;
 
   scalar = (code2 == TYPE_CODE_INT || code2 == TYPE_CODE_FLT
@@ -180,7 +216,7 @@ value_cast (type, arg2)
 	 type of the target as a superclass.  If so, we'll need to
 	 offset the object in addition to changing its type.  */
       value_ptr v = search_struct_field (type_name_no_tag (type),
-					 arg2, 0, VALUE_TYPE (arg2), 1);
+					 arg2, 0, type2, 1);
       if (v)
 	{
 	  VALUE_TYPE (v) = type;
@@ -193,15 +229,15 @@ value_cast (type, arg2)
 	    || code1 == TYPE_CODE_RANGE)
 	   && (scalar || code2 == TYPE_CODE_PTR))
     return value_from_longest (type, value_as_long (arg2));
-  else if (TYPE_LENGTH (type) == TYPE_LENGTH (VALUE_TYPE (arg2)))
+  else if (TYPE_LENGTH (type) == TYPE_LENGTH (type2))
     {
       if (code1 == TYPE_CODE_PTR && code2 == TYPE_CODE_PTR)
 	{
 	  /* Look in the type of the source to see if it contains the
 	     type of the target as a superclass.  If so, we'll need to
 	     offset the pointer rather than just change its type.  */
-	  struct type *t1 = TYPE_TARGET_TYPE (type);
-	  struct type *t2 = TYPE_TARGET_TYPE (VALUE_TYPE (arg2));
+	  struct type *t1 = check_typedef (TYPE_TARGET_TYPE (type));
+	  struct type *t2 = check_typedef (TYPE_TARGET_TYPE (type2));
 	  if (   TYPE_CODE (t1) == TYPE_CODE_STRUCT
 	      && TYPE_CODE (t2) == TYPE_CODE_STRUCT
 	      && TYPE_NAME (t1) != 0) /* if name unknown, can't have supercl */
@@ -225,19 +261,26 @@ value_cast (type, arg2)
       struct type *range1, *range2, *eltype1, *eltype2;
       value_ptr val;
       int count1, count2;
+      LONGEST low_bound, high_bound;
       char *valaddr, *valaddr_data;
       if (code2 == TYPE_CODE_BITSTRING)
 	error ("not implemented: converting bitstring to varying type");
       if ((code2 != TYPE_CODE_ARRAY && code2 != TYPE_CODE_STRING)
-	  || (eltype1 = TYPE_TARGET_TYPE (TYPE_FIELD_TYPE (type, 1)),
-	      eltype2 = TYPE_TARGET_TYPE (VALUE_TYPE (arg2)),
+	  || (eltype1 = check_typedef (TYPE_TARGET_TYPE (TYPE_FIELD_TYPE (type, 1))),
+	      eltype2 = check_typedef (TYPE_TARGET_TYPE (type2)),
 	      (TYPE_LENGTH (eltype1) != TYPE_LENGTH (eltype2)
 	       /* || TYPE_CODE (eltype1) != TYPE_CODE (eltype2) */ )))
 	error ("Invalid conversion to varying type");
       range1 = TYPE_FIELD_TYPE (TYPE_FIELD_TYPE (type, 1), 0);
-      range2 = TYPE_FIELD_TYPE (VALUE_TYPE (arg2), 0);
-      count1 = TYPE_HIGH_BOUND (range1) - TYPE_LOW_BOUND (range1) + 1;
-      count2 = TYPE_HIGH_BOUND (range2) - TYPE_LOW_BOUND (range2) + 1;
+      range2 = TYPE_FIELD_TYPE (type2, 0);
+      if (get_discrete_bounds (range1, &low_bound, &high_bound) < 0)
+	count1 = -1;
+      else
+	count1 = high_bound - low_bound + 1;
+      if (get_discrete_bounds (range2, &low_bound, &high_bound) < 0)
+	count1 = -1, count2 = 0;  /* To force error before */
+      else
+	count2 = high_bound - low_bound + 1;
       if (count2 > count1)
 	error ("target varying type is too small");
       val = allocate_value (type);
@@ -278,7 +321,7 @@ value_zero (type, lv)
 {
   register value_ptr val = allocate_value (type);
 
-  memset (VALUE_CONTENTS (val), 0, TYPE_LENGTH (type));
+  memset (VALUE_CONTENTS (val), 0, TYPE_LENGTH (check_typedef (type)));
   VALUE_LVAL (val) = lv;
 
   return val;
@@ -300,7 +343,7 @@ value_at (type, addr)
 {
   register value_ptr val;
 
-  if (TYPE_CODE (type) == TYPE_CODE_VOID)
+  if (TYPE_CODE (check_typedef (type)) == TYPE_CODE_VOID)
     error ("Attempt to dereference a generic pointer.");
 
   val = allocate_value (type);
@@ -322,7 +365,7 @@ value_at_lazy (type, addr)
 {
   register value_ptr val;
 
-  if (TYPE_CODE (type) == TYPE_CODE_VOID)
+  if (TYPE_CODE (check_typedef (type)) == TYPE_CODE_VOID)
     error ("Attempt to dereference a generic pointer.");
 
   val = allocate_value (type);
@@ -351,10 +394,10 @@ value_fetch_lazy (val)
      register value_ptr val;
 {
   CORE_ADDR addr = VALUE_ADDRESS (val) + VALUE_OFFSET (val);
+  int length = TYPE_LENGTH (VALUE_TYPE (val));
 
-  if (TYPE_LENGTH (VALUE_TYPE (val)))
-    read_memory (addr, VALUE_CONTENTS_RAW (val), 
-	         TYPE_LENGTH (VALUE_TYPE (val)));
+  if (length)
+    read_memory (addr, VALUE_CONTENTS_RAW (val), length);
   VALUE_LAZY (val) = 0;
   return 0;
 }
@@ -375,12 +418,14 @@ value_assign (toval, fromval)
   if (!toval->modifiable)
     error ("Left operand of assignment is not a modifiable lvalue.");
 
-  COERCE_ARRAY (fromval);
   COERCE_REF (toval);
 
   type = VALUE_TYPE (toval);
   if (VALUE_LVAL (toval) != lval_internalvar)
     fromval = value_cast (type, fromval);
+  else
+    COERCE_ARRAY (fromval);
+  CHECK_TYPEDEF (type);
 
   /* If TOVAL is a special machine register requiring conversion
      of program values to a special raw format,
@@ -394,7 +439,8 @@ value_assign (toval, fromval)
       int regno = VALUE_REGNO (toval);
       if (REGISTER_CONVERTIBLE (regno))
 	{
-	  REGISTER_CONVERT_TO_RAW (VALUE_TYPE (fromval), regno,
+	  struct type *fromtype = check_typedef (VALUE_TYPE (fromval));
+	  REGISTER_CONVERT_TO_RAW (fromtype, regno,
 				   VALUE_CONTENTS (fromval), raw_buffer);
 	  use_buffer = REGISTER_RAW_SIZE (regno);
 	}
@@ -405,7 +451,7 @@ value_assign (toval, fromval)
     {
     case lval_internalvar:
       set_internalvar (VALUE_INTERNALVAR (toval), fromval);
-      break;
+      return value_copy (VALUE_INTERNALVAR (toval)->value);
 
     case lval_internalvar_component:
       set_internalvar_component (VALUE_INTERNALVAR (toval),
@@ -426,7 +472,7 @@ value_assign (toval, fromval)
 		     + HOST_CHAR_BIT - 1)
 		    / HOST_CHAR_BIT;
 
-	  if (len > sizeof (LONGEST))
+	  if (len > (int) sizeof (LONGEST))
 	    error ("Can't handle bitfields which don't fit in a %d bit word.",
 		   sizeof (LONGEST) * HOST_CHAR_BIT);
 
@@ -451,7 +497,7 @@ value_assign (toval, fromval)
 	  char buffer[sizeof (LONGEST)];
           int len = REGISTER_RAW_SIZE (VALUE_REGNO (toval));
 
-	  if (len > sizeof (LONGEST))
+	  if (len > (int) sizeof (LONGEST))
 	    error ("Can't handle bitfields in registers larger than %d bits.",
 		   sizeof (LONGEST) * HOST_CHAR_BIT);
 
@@ -577,17 +623,22 @@ Can't handle bitfield which doesn't fit in a single register.");
       error ("Left operand of assignment is not an lvalue.");
     }
 
-  /* Return a value just like TOVAL except with the contents of FROMVAL
-     (except in the case of the type if TOVAL is an internalvar).  */
-
-  if (VALUE_LVAL (toval) == lval_internalvar
-      || VALUE_LVAL (toval) == lval_internalvar_component)
+  /* If the field does not entirely fill a LONGEST, then zero the sign bits.
+     If the field is signed, and is negative, then sign extend. */
+  if ((VALUE_BITSIZE (toval) > 0)
+      && (VALUE_BITSIZE (toval) < 8 * (int) sizeof (LONGEST)))
     {
-      type = VALUE_TYPE (fromval);
+      LONGEST fieldval = value_as_long (fromval);
+      LONGEST valmask = (((unsigned LONGEST) 1) << VALUE_BITSIZE (toval)) - 1;
+
+      fieldval &= valmask;
+      if (!TYPE_UNSIGNED (type) && (fieldval & (valmask ^ (valmask >> 1))))
+	fieldval |= ~valmask;
+
+      fromval = value_from_longest (type, fieldval);
     }
 
-  val = allocate_value (type);
-  memcpy (val, toval, VALUE_CONTENTS_RAW (val) - (char *) val);
+  val = value_copy (toval);
   memcpy (VALUE_CONTENTS_RAW (val), VALUE_CONTENTS (fromval),
 	  TYPE_LENGTH (type));
   VALUE_TYPE (val) = type;
@@ -608,14 +659,12 @@ value_repeat (arg1, count)
     error ("Only values in memory can be extended with '@'.");
   if (count < 1)
     error ("Invalid number %d of repetitions.", count);
-  if (VALUE_REPEATED (arg1))
-    error ("Cannot create artificial arrays of artificial arrays.");
 
   val = allocate_repeat_value (VALUE_TYPE (arg1), count);
 
   read_memory (VALUE_ADDRESS (arg1) + VALUE_OFFSET (arg1),
 	       VALUE_CONTENTS_RAW (val),
-	       TYPE_LENGTH (VALUE_TYPE (val)) * count);
+	       TYPE_LENGTH (VALUE_TYPE (val)));
   VALUE_LVAL (val) = lval_memory;
   VALUE_ADDRESS (val) = VALUE_ADDRESS (arg1) + VALUE_OFFSET (arg1);
 
@@ -679,21 +728,12 @@ value_ptr
 value_coerce_array (arg1)
      value_ptr arg1;
 {
-  register struct type *type;
+  register struct type *type = check_typedef (VALUE_TYPE (arg1));
 
   if (VALUE_LVAL (arg1) != lval_memory)
     error ("Attempt to take address of value not located in memory.");
 
-  /* Get type of elements.  */
-  if (TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_ARRAY
-      || TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_STRING)
-    type = TYPE_TARGET_TYPE (VALUE_TYPE (arg1));
-  else
-    /* A phony array made by value_repeat.
-       Its type is the type of the elements, not an array type.  */
-    type = VALUE_TYPE (arg1);
-
-  return value_from_longest (lookup_pointer_type (type),
+  return value_from_longest (lookup_pointer_type (TYPE_TARGET_TYPE (type)),
 		       (LONGEST) (VALUE_ADDRESS (arg1) + VALUE_OFFSET (arg1)));
 }
 
@@ -718,7 +758,7 @@ value_ptr
 value_addr (arg1)
      value_ptr arg1;
 {
-  struct type *type = VALUE_TYPE (arg1);
+  struct type *type = check_typedef (VALUE_TYPE (arg1));
   if (TYPE_CODE (type) == TYPE_CODE_REF)
     {
       /* Copy the value, but change the type from (T&) to (T*).
@@ -734,7 +774,7 @@ value_addr (arg1)
   if (VALUE_LVAL (arg1) != lval_memory)
     error ("Attempt to take address of value not located in memory.");
 
-  return value_from_longest (lookup_pointer_type (type),
+  return value_from_longest (lookup_pointer_type (VALUE_TYPE (arg1)),
 		(LONGEST) (VALUE_ADDRESS (arg1) + VALUE_OFFSET (arg1)));
 }
 
@@ -744,21 +784,22 @@ value_ptr
 value_ind (arg1)
      value_ptr arg1;
 {
+  struct type *type1;
   COERCE_ARRAY (arg1);
+  type1 = check_typedef (VALUE_TYPE (arg1));
 
-  if (TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_MEMBER)
+  if (TYPE_CODE (type1) == TYPE_CODE_MEMBER)
     error ("not implemented: member types in value_ind");
 
   /* Allow * on an integer so we can cast it to whatever we want.
      This returns an int, which seems like the most C-like thing
      to do.  "long long" variables are rare enough that
      BUILTIN_TYPE_LONGEST would seem to be a mistake.  */
-  if (TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_INT)
+  if (TYPE_CODE (type1) == TYPE_CODE_INT)
     return value_at (builtin_type_int,
 		     (CORE_ADDR) value_as_long (arg1));
-  else if (TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_PTR)
-    return value_at_lazy (TYPE_TARGET_TYPE (VALUE_TYPE (arg1)),
-			  value_as_pointer (arg1));
+  else if (TYPE_CODE (type1) == TYPE_CODE_PTR)
+    return value_at_lazy (TYPE_TARGET_TYPE (type1), value_as_pointer (arg1));
   error ("Attempt to take contents of a non-pointer value.");
   return 0;  /* For lint -- never reached */
 }
@@ -836,21 +877,14 @@ value_arg_coerce (arg, param_type)
      value_ptr arg;
      struct type *param_type;
 {
-  register struct type *type;
-
-#if 1	/* FIXME:  This is only a temporary patch.  -fnf */
-  if (current_language->c_style_arrays
-      && (VALUE_REPEATED (arg)
-	  || TYPE_CODE (VALUE_TYPE (arg)) == TYPE_CODE_ARRAY))
-    arg = value_coerce_array (arg);
-#endif
-
-  type = param_type ? param_type : VALUE_TYPE (arg);
+  register struct type *arg_type = check_typedef (VALUE_TYPE (arg));
+  register struct type *type
+    = param_type ? check_typedef (param_type) : arg_type;
 
   switch (TYPE_CODE (type))
     {
     case TYPE_CODE_REF:
-      if (TYPE_CODE (VALUE_TYPE (arg)) != TYPE_CODE_REF)
+      if (TYPE_CODE (arg_type) != TYPE_CODE_REF)
 	{
 	  arg = value_addr (arg);
 	  VALUE_TYPE (arg) = param_type;
@@ -864,16 +898,25 @@ value_arg_coerce (arg, param_type)
       if (TYPE_LENGTH (type) < TYPE_LENGTH (builtin_type_int))
 	type = builtin_type_int;
       break;
-    case TYPE_CODE_FLT:
-      if (TYPE_LENGTH (type) < TYPE_LENGTH (builtin_type_double))
-	type = builtin_type_double;
-      break;
+   case TYPE_CODE_FLT:
+     /* coerce float to double, unless the function prototype specifies float */
+     if (COERCE_FLOAT_TO_DOUBLE)
+       {
+	 if (TYPE_LENGTH (type) < TYPE_LENGTH (builtin_type_double))
+	   type = builtin_type_double;
+	 else if (TYPE_LENGTH (type) > TYPE_LENGTH (builtin_type_double))
+	   type = builtin_type_long_double;
+       }
+     break;
     case TYPE_CODE_FUNC:
       type = lookup_pointer_type (type);
       break;
+    case TYPE_CODE_ARRAY:
+      if (current_language->c_style_arrays)
+	type = lookup_pointer_type (TYPE_TARGET_TYPE (type));
+      break;
     case TYPE_CODE_UNDEF:
     case TYPE_CODE_PTR:
-    case TYPE_CODE_ARRAY:
     case TYPE_CODE_STRUCT:
     case TYPE_CODE_UNION:
     case TYPE_CODE_VOID:
@@ -900,7 +943,7 @@ find_function_addr (function, retval_type)
      value_ptr function;
      struct type **retval_type;
 {
-  register struct type *ftype = VALUE_TYPE (function);
+  register struct type *ftype = check_typedef (VALUE_TYPE (function));
   register enum type_code code = TYPE_CODE (ftype);
   struct type *value_type;
   CORE_ADDR funaddr;
@@ -917,8 +960,9 @@ find_function_addr (function, retval_type)
   else if (code == TYPE_CODE_PTR)
     {
       funaddr = value_as_pointer (function);
-      if (TYPE_CODE (TYPE_TARGET_TYPE (ftype)) == TYPE_CODE_FUNC
-	  || TYPE_CODE (TYPE_TARGET_TYPE (ftype)) == TYPE_CODE_METHOD)
+      ftype = check_typedef (TYPE_TARGET_TYPE (ftype));
+      if (TYPE_CODE (ftype) == TYPE_CODE_FUNC
+	  || TYPE_CODE (ftype) == TYPE_CODE_METHOD)
 	{
 #ifdef CONVERT_FROM_FUNC_PTR_ADDR
 	  /* FIXME: This is a workaround for the unusual function
@@ -926,7 +970,7 @@ find_function_addr (function, retval_type)
 	     in config/rs6000/tm-rs6000.h  */
 	  funaddr = CONVERT_FROM_FUNC_PTR_ADDR (funaddr);
 #endif
-	  value_type = TYPE_TARGET_TYPE (TYPE_TARGET_TYPE (ftype));
+	  value_type = TYPE_TARGET_TYPE (ftype);
 	}
       else
 	value_type = builtin_type_int;
@@ -993,7 +1037,7 @@ call_function_by_hand (function, nargs, args)
   CORE_ADDR funaddr;
   int using_gcc;
   CORE_ADDR real_pc;
-  struct type *ftype = SYMBOL_TYPE (function);
+  struct type *ftype = check_typedef (SYMBOL_TYPE (function));
 
   if (!target_has_execution)
     noprocess();
@@ -1017,6 +1061,7 @@ call_function_by_hand (function, nargs, args)
 #endif
 
   funaddr = find_function_addr (function, &value_type);
+  CHECK_TYPEDEF (value_type);
 
   {
     struct block *b = block_for_pc (funaddr);
@@ -1032,7 +1077,7 @@ call_function_by_hand (function, nargs, args)
 
   /* Create a call sequence customized for this function
      and the number of arguments for it.  */
-  for (i = 0; i < sizeof dummy / sizeof (dummy[0]); i++)
+  for (i = 0; i < (int) (sizeof (dummy) / sizeof (dummy[0])); i++)
     store_unsigned_integer (&dummy1[i * REGISTER_SIZE],
 			    REGISTER_SIZE,
 			    (unsigned LONGEST)dummy[i]);
@@ -1105,40 +1150,48 @@ call_function_by_hand (function, nargs, args)
     /* This is a machine like the sparc, where we may need to pass a pointer
        to the structure, not the structure itself.  */
     for (i = nargs - 1; i >= 0; i--)
-      if ((TYPE_CODE (VALUE_TYPE (args[i])) == TYPE_CODE_STRUCT
-	   || TYPE_CODE (VALUE_TYPE (args[i])) == TYPE_CODE_UNION
-	   || TYPE_CODE (VALUE_TYPE (args[i])) == TYPE_CODE_ARRAY
-	   || TYPE_CODE (VALUE_TYPE (args[i])) == TYPE_CODE_STRING)
-	  && REG_STRUCT_HAS_ADDR (using_gcc, VALUE_TYPE (args[i])))
-	{
-	  CORE_ADDR addr;
-	  int len = TYPE_LENGTH (VALUE_TYPE (args[i]));
+      {
+	struct type *arg_type = check_typedef (VALUE_TYPE (args[i]));
+	if ((TYPE_CODE (arg_type) == TYPE_CODE_STRUCT
+	     || TYPE_CODE (arg_type) == TYPE_CODE_UNION
+	     || TYPE_CODE (arg_type) == TYPE_CODE_ARRAY
+	     || TYPE_CODE (arg_type) == TYPE_CODE_STRING
+	     || TYPE_CODE (arg_type) == TYPE_CODE_BITSTRING
+	     || TYPE_CODE (arg_type) == TYPE_CODE_SET
+	     || (TYPE_CODE (arg_type) == TYPE_CODE_FLT
+		 && TYPE_LENGTH (arg_type) > 8)
+	     )
+	  && REG_STRUCT_HAS_ADDR (using_gcc, arg_type))
+	  {
+	    CORE_ADDR addr;
+	    int len = TYPE_LENGTH (arg_type);
 #ifdef STACK_ALIGN
-	  int aligned_len = STACK_ALIGN (len);
+	    int aligned_len = STACK_ALIGN (len);
 #else
-	  int aligned_len = len;
+	    int aligned_len = len;
 #endif
 #if !(1 INNER_THAN 2)
-	  /* The stack grows up, so the address of the thing we push
-	     is the stack pointer before we push it.  */
-	  addr = sp;
+	    /* The stack grows up, so the address of the thing we push
+	       is the stack pointer before we push it.  */
+	    addr = sp;
 #else
-	  sp -= aligned_len;
+	    sp -= aligned_len;
 #endif
-	  /* Push the structure.  */
-	  write_memory (sp, VALUE_CONTENTS (args[i]), len);
+	    /* Push the structure.  */
+	    write_memory (sp, VALUE_CONTENTS (args[i]), len);
 #if 1 INNER_THAN 2
-	  /* The stack grows down, so the address of the thing we push
-	     is the stack pointer after we push it.  */
-	  addr = sp;
+	    /* The stack grows down, so the address of the thing we push
+	       is the stack pointer after we push it.  */
+	    addr = sp;
 #else
-	  sp += aligned_len;
+	    sp += aligned_len;
 #endif
-	  /* The value we're going to pass is the address of the thing
-	     we just pushed.  */
-	  args[i] = value_from_longest (lookup_pointer_type (value_type),
-					(LONGEST) addr);
-	}
+	    /* The value we're going to pass is the address of the thing
+	       we just pushed.  */
+	    args[i] = value_from_longest (lookup_pointer_type (value_type),
+					  (LONGEST) addr);
+	  }
+      }
   }
 #endif /* REG_STRUCT_HAS_ADDR.  */
 
@@ -1308,7 +1361,7 @@ value_array (lowbound, highbound, elemvec)
 {
   int nelem;
   int idx;
-  int typelength;
+  unsigned int typelength;
   value_ptr val;
   struct type *rangetype;
   struct type *arraytype;
@@ -1323,7 +1376,7 @@ value_array (lowbound, highbound, elemvec)
       error ("bad array bounds (%d, %d)", lowbound, highbound);
     }
   typelength = TYPE_LENGTH (VALUE_TYPE (elemvec[0]));
-  for (idx = 0; idx < nelem; idx++)
+  for (idx = 1; idx < nelem; idx++)
     {
       if (TYPE_LENGTH (VALUE_TYPE (elemvec[idx])) != typelength)
 	{
@@ -1331,6 +1384,23 @@ value_array (lowbound, highbound, elemvec)
 	}
     }
 
+  rangetype = create_range_type ((struct type *) NULL, builtin_type_int,
+				 lowbound, highbound);
+  arraytype = create_array_type ((struct type *) NULL, 
+				 VALUE_TYPE (elemvec[0]), rangetype);
+
+  if (!current_language->c_style_arrays)
+    {
+      val = allocate_value (arraytype);
+      for (idx = 0; idx < nelem; idx++)
+	{
+	  memcpy (VALUE_CONTENTS_RAW (val) + (idx * typelength),
+		  VALUE_CONTENTS (elemvec[idx]),
+		  typelength);
+	}
+      return val;
+    }
+
   /* Allocate space to store the array in the inferior, and then initialize
      it by copying in each element.  FIXME:  Is it worth it to create a
      local buffer in which to collect each value and then write all the
@@ -1345,10 +1415,6 @@ value_array (lowbound, highbound, elemvec)
 
   /* Create the array type and set up an array value to be evaluated lazily. */
 
-  rangetype = create_range_type ((struct type *) NULL, builtin_type_int,
-				 lowbound, highbound);
-  arraytype = create_array_type ((struct type *) NULL, 
-				 VALUE_TYPE (elemvec[0]), rangetype);
   val = value_at_lazy (arraytype, addr);
   return (val);
 }
@@ -1404,7 +1470,7 @@ value_bitstring (ptr, len)
   struct type *type = create_set_type ((struct type*) NULL, domain_type);
   TYPE_CODE (type) = TYPE_CODE_BITSTRING;
   val = allocate_value (type);
-  memcpy (VALUE_CONTENTS_RAW (val), ptr, TYPE_LENGTH (type) / TARGET_CHAR_BIT);
+  memcpy (VALUE_CONTENTS_RAW (val), ptr, TYPE_LENGTH (type));
   return val;
 }
 
@@ -1444,13 +1510,13 @@ typecmp (staticp, t1, t2)
     struct type *tt1, *tt2;
       if (! t2[i])
 	return i+1;
-      tt1 = t1[i];
-      tt2 = VALUE_TYPE(t2[i]);
+      tt1 = check_typedef (t1[i]);
+      tt2 = check_typedef (VALUE_TYPE(t2[i]));
       if (TYPE_CODE (tt1) == TYPE_CODE_REF
 	  /* We should be doing hairy argument matching, as below.  */
-	  && (TYPE_CODE (TYPE_TARGET_TYPE (tt1)) == TYPE_CODE (tt2)))
+	  && (TYPE_CODE (check_typedef (TYPE_TARGET_TYPE (tt1))) == TYPE_CODE (tt2)))
 	{
-	  if (TYPE_CODE (tt2) == TYPE_CODE_ARRAY || VALUE_REPEATED (t2[i]))
+	  if (TYPE_CODE (tt2) == TYPE_CODE_ARRAY)
 	    t2[i] = value_coerce_array (t2[i]);
 	  else
 	    t2[i] = value_addr (t2[i]);
@@ -1458,10 +1524,11 @@ typecmp (staticp, t1, t2)
 	}
 
       while (TYPE_CODE (tt1) == TYPE_CODE_PTR
-	  && (TYPE_CODE(tt2)==TYPE_CODE_ARRAY || TYPE_CODE(tt2)==TYPE_CODE_PTR))
+	  && (   TYPE_CODE (tt2) == TYPE_CODE_ARRAY
+	      || TYPE_CODE (tt2) == TYPE_CODE_PTR))
 	{
-	   tt1 = TYPE_TARGET_TYPE(tt1); 
-	   tt2 = TYPE_TARGET_TYPE(tt2);
+	   tt1 = check_typedef (TYPE_TARGET_TYPE(tt1)); 
+	   tt2 = check_typedef (TYPE_TARGET_TYPE(tt2));
 	}
       if (TYPE_CODE(tt1) == TYPE_CODE(tt2)) continue;
       /* Array to pointer is a `trivial conversion' according to the ARM.  */
@@ -1494,7 +1561,7 @@ search_struct_field (name, arg1, offset, type, looking_for_baseclass)
 {
   int i;
 
-  check_stub_type (type);
+  CHECK_TYPEDEF (type);
 
   if (! looking_for_baseclass)
     for (i = TYPE_NFIELDS (type) - 1; i >= TYPE_N_BASECLASSES (type); i--)
@@ -1521,22 +1588,50 @@ search_struct_field (name, arg1, offset, type, looking_for_baseclass)
 	      error("there is no field named %s", name);
 	    return v;
 	  }
-	if (t_field_name && t_field_name[0] == '\0'
-	    && TYPE_CODE (TYPE_FIELD_TYPE (type, i)) == TYPE_CODE_UNION)
+
+	if (t_field_name
+	    && (t_field_name[0] == '\0'
+		|| (TYPE_CODE (type) == TYPE_CODE_UNION
+		    && STREQ (t_field_name, "else"))))
 	  {
-	    /* Look for a match through the fields of an anonymous union.  */
-	    value_ptr v;
-	    v = search_struct_field (name, arg1, offset,
-				     TYPE_FIELD_TYPE (type, i),
-				     looking_for_baseclass);
-	    if (v)
-	      return v;
+	    struct type *field_type = TYPE_FIELD_TYPE (type, i);
+	    if (TYPE_CODE (field_type) == TYPE_CODE_UNION
+		|| TYPE_CODE (field_type) == TYPE_CODE_STRUCT)
+	      {
+		/* Look for a match through the fields of an anonymous union,
+		   or anonymous struct.  C++ provides anonymous unions.
+
+		   In the GNU Chill implementation of variant record types,
+		   each <alternative field> has an (anonymous) union type,
+		   each member of the union represents a <variant alternative>.
+		   Each <variant alternative> is represented as a struct,
+		   with a member for each <variant field>.  */
+		   
+		value_ptr v;
+		int new_offset = offset;
+
+		/* This is pretty gross.  In G++, the offset in an anonymous
+		   union is relative to the beginning of the enclosing struct.
+		   In the GNU Chill implementation of variant records,
+		   the bitpos is zero in an anonymous union field, so we
+		   have to add the offset of the union here. */
+		if (TYPE_CODE (field_type) == TYPE_CODE_STRUCT
+		    || (TYPE_NFIELDS (field_type) > 0
+			&& TYPE_FIELD_BITPOS (field_type, 0) == 0))
+		  new_offset += TYPE_FIELD_BITPOS (type, i) / 8;
+
+		v = search_struct_field (name, arg1, new_offset, field_type,
+					 looking_for_baseclass);
+		if (v)
+		  return v;
+	      }
 	  }
       }
 
   for (i = TYPE_N_BASECLASSES (type) - 1; i >= 0; i--)
     {
       value_ptr v;
+      struct type *basetype = check_typedef (TYPE_BASECLASS (type, i));
       /* If we are looking for baseclasses, this is what we get when we
 	 hit them.  But it could happen that the base part's member name
 	 is not yet filled in.  */
@@ -1546,15 +1641,28 @@ search_struct_field (name, arg1, offset, type, looking_for_baseclass)
 
       if (BASETYPE_VIA_VIRTUAL (type, i))
 	{
-	  value_ptr v2;
-	  /* Fix to use baseclass_offset instead. FIXME */
-	  baseclass_addr (type, i, VALUE_CONTENTS (arg1) + offset,
-			  &v2, (int *)NULL);
-	  if (v2 == 0)
+	  int boffset = VALUE_OFFSET (arg1) + offset;
+	  boffset = baseclass_offset (type, i,
+				      VALUE_CONTENTS (arg1) + boffset,
+				      VALUE_ADDRESS (arg1) + boffset);
+	  if (boffset == -1)
 	    error ("virtual baseclass botch");
 	  if (found_baseclass)
-	    return v2;
-	  v = search_struct_field (name, v2, 0, TYPE_BASECLASS (type, i),
+	    {
+	      value_ptr v2 = allocate_value (basetype);
+	      VALUE_LVAL (v2) = VALUE_LVAL (arg1);
+	      VALUE_ADDRESS (v2) = VALUE_ADDRESS (arg1);
+	      VALUE_OFFSET (v2) = VALUE_OFFSET (arg1) + offset + boffset;
+	      if (VALUE_LAZY (arg1))
+		VALUE_LAZY (v2) = 1;
+	      else
+		memcpy (VALUE_CONTENTS_RAW (v2),
+			VALUE_CONTENTS_RAW (arg1) + offset + boffset,
+			TYPE_LENGTH (basetype));
+	      return v2;
+	    }
+	  v = search_struct_field (name, arg1, offset + boffset,
+				   TYPE_BASECLASS (type, i),
 				   looking_for_baseclass);
 	}
       else if (found_baseclass)
@@ -1562,8 +1670,7 @@ search_struct_field (name, arg1, offset, type, looking_for_baseclass)
       else
 	v = search_struct_field (name, arg1,
 				 offset + TYPE_BASECLASS_BITPOS (type, i) / 8,
-				 TYPE_BASECLASS (type, i),
-				 looking_for_baseclass);
+				 basetype, looking_for_baseclass);
       if (v) return v;
     }
   return NULL;
@@ -1587,7 +1694,7 @@ search_struct_method (name, arg1p, args, offset, static_memfuncp, type)
   int name_matched = 0;
   char dem_opname[64];
 
-  check_stub_type (type);
+  CHECK_TYPEDEF (type);
   for (i = TYPE_NFN_FIELDS (type) - 1; i >= 0; i--)
     {
       char *t_field_name = TYPE_FN_FIELDLIST_NAME (type, i);
@@ -1633,7 +1740,11 @@ search_struct_method (name, arg1p, args, offset, static_memfuncp, type)
 
       if (BASETYPE_VIA_VIRTUAL (type, i))
 	{
-	  base_offset = baseclass_offset (type, i, *arg1p, offset);
+	  base_offset = VALUE_OFFSET (*arg1p) + offset;
+	  base_offset =
+	    baseclass_offset (type, i,
+			      VALUE_CONTENTS (*arg1p) + base_offset,
+			      VALUE_ADDRESS (*arg1p) + base_offset);
 	  if (base_offset == -1)
 	    error ("virtual baseclass botch");
 	}
@@ -1684,7 +1795,7 @@ value_struct_elt (argp, args, name, static_memfuncp, err)
 
   COERCE_ARRAY (*argp);
 
-  t = VALUE_TYPE (*argp);
+  t = check_typedef (VALUE_TYPE (*argp));
 
   /* Follow pointers until we get to a non-pointer.  */
 
@@ -1694,7 +1805,7 @@ value_struct_elt (argp, args, name, static_memfuncp, err)
       /* Don't coerce fn pointer to fn and then back again!  */
       if (TYPE_CODE (VALUE_TYPE (*argp)) != TYPE_CODE_FUNC)
 	COERCE_ARRAY (*argp);
-      t = VALUE_TYPE (*argp);
+      t = check_typedef (VALUE_TYPE (*argp));
     }
 
   if (TYPE_CODE (t) == TYPE_CODE_MEMBER)
@@ -1787,7 +1898,7 @@ destructor_name_p (name, type)
     {
       char *dname = type_name_no_tag (type);
       char *cp = strchr (dname, '<');
-      int len;
+      unsigned int len;
 
       /* Do not compare the template part for template classes.  */
       if (cp == NULL)
@@ -1858,8 +1969,13 @@ check_field (arg1, name)
 
   /* Follow pointers until we get to a non-pointer.  */
 
-  while (TYPE_CODE (t) == TYPE_CODE_PTR || TYPE_CODE (t) == TYPE_CODE_REF)
-    t = TYPE_TARGET_TYPE (t);
+  for (;;)
+    {
+      CHECK_TYPEDEF (t);
+      if (TYPE_CODE (t) != TYPE_CODE_PTR && TYPE_CODE (t) != TYPE_CODE_REF)
+	break;
+      t = TYPE_TARGET_TYPE (t);
+    }
 
   if (TYPE_CODE (t) == TYPE_CODE_MEMBER)
     error ("not implemented: member type in check_field");
@@ -2075,33 +2191,63 @@ value_slice (array, lowbound, length)
      value_ptr array;
      int lowbound, length;
 {
-  COERCE_VARYING_ARRAY (array);
-  if (TYPE_CODE (VALUE_TYPE (array)) == TYPE_CODE_BITSTRING)
-    error ("not implemented - bitstring slice");
-  if (TYPE_CODE (VALUE_TYPE (array)) != TYPE_CODE_ARRAY
-      && TYPE_CODE (VALUE_TYPE (array)) != TYPE_CODE_STRING)
+  struct type *slice_range_type, *slice_type, *range_type;
+  LONGEST lowerbound, upperbound, offset;
+  value_ptr slice;
+  struct type *array_type;
+  array_type = check_typedef (VALUE_TYPE (array));
+  COERCE_VARYING_ARRAY (array, array_type);
+  if (TYPE_CODE (array_type) != TYPE_CODE_ARRAY
+      && TYPE_CODE (array_type) != TYPE_CODE_STRING
+      && TYPE_CODE (array_type) != TYPE_CODE_BITSTRING)
     error ("cannot take slice of non-array");
+  range_type = TYPE_INDEX_TYPE (array_type);
+  if (get_discrete_bounds (range_type, &lowerbound, &upperbound) < 0)
+    error ("slice from bad array or bitstring");
+  if (lowbound < lowerbound || length < 0
+      || lowbound + length - 1 > upperbound
+      /* Chill allows zero-length strings but not arrays. */
+      || (current_language->la_language == language_chill
+	  && length == 0 && TYPE_CODE (array_type) == TYPE_CODE_ARRAY))
+    error ("slice out of range");
+  /* FIXME-type-allocation: need a way to free this type when we are
+     done with it.  */
+  slice_range_type = create_range_type ((struct type*) NULL,
+					TYPE_TARGET_TYPE (range_type),
+					lowerbound, lowerbound + length - 1);
+  if (TYPE_CODE (array_type) == TYPE_CODE_BITSTRING)
+    {
+      int i;
+      slice_type = create_set_type ((struct type*) NULL, slice_range_type);
+      TYPE_CODE (slice_type) = TYPE_CODE_BITSTRING;
+      slice = value_zero (slice_type, not_lval);
+      for (i = 0; i < length; i++)
+	{
+	  int element = value_bit_index (array_type,
+					 VALUE_CONTENTS (array),
+					 lowbound + i);
+	  if (element < 0)
+	    error ("internal error accessing bitstring");
+	  else if (element > 0)
+	    {
+	      int j = i % TARGET_CHAR_BIT;
+	      if (BITS_BIG_ENDIAN)
+		j = TARGET_CHAR_BIT - 1 - j;
+	      VALUE_CONTENTS_RAW (slice)[i / TARGET_CHAR_BIT] |= (1 << j);
+	    }
+	}
+      /* We should set the address, bitssize, and bitspos, so the clice
+	 can be used on the LHS, but that may require extensions to
+	 value_assign.  For now, just leave as a non_lval.  FIXME.  */
+    }
   else
     {
-      struct type *slice_range_type, *slice_type;
-      value_ptr slice;
-      struct type *range_type = TYPE_FIELD_TYPE (VALUE_TYPE (array), 0);
-      struct type *element_type = TYPE_TARGET_TYPE (VALUE_TYPE (array));
-      int lowerbound = TYPE_LOW_BOUND (range_type);
-      int upperbound = TYPE_HIGH_BOUND (range_type);
-      int offset = (lowbound - lowerbound) * TYPE_LENGTH (element_type);
-      if (lowbound < lowerbound || length < 0
-	  || lowbound + length - 1 > upperbound)
-	error ("slice out of range");
-      /* FIXME-type-allocation: need a way to free this type when we are
-	 done with it.  */
-      slice_range_type = create_range_type ((struct type*) NULL,
-					    TYPE_TARGET_TYPE (range_type),
-					    lowerbound,
-					    lowerbound + length - 1);
+      struct type *element_type = TYPE_TARGET_TYPE (array_type);
+      offset
+	= (lowbound - lowerbound) * TYPE_LENGTH (check_typedef (element_type));
       slice_type = create_array_type ((struct type*) NULL, element_type,
 				      slice_range_type);
-      TYPE_CODE (slice_type) = TYPE_CODE (VALUE_TYPE (array));
+      TYPE_CODE (slice_type) = TYPE_CODE (array_type);
       slice = allocate_value (slice_type);
       if (VALUE_LAZY (array))
 	VALUE_LAZY (slice) = 1;
@@ -2114,8 +2260,8 @@ value_slice (array, lowbound, length)
 	VALUE_LVAL (slice) = VALUE_LVAL (array);
       VALUE_ADDRESS (slice) = VALUE_ADDRESS (array);
       VALUE_OFFSET (slice) = VALUE_OFFSET (array) + offset;
-      return slice;
     }
+  return slice;
 }
 
 /* Assuming chill_varying_type (VARRAY) is true, return an equivalent
@@ -2125,7 +2271,7 @@ value_ptr
 varying_to_slice (varray)
      value_ptr varray;
 {
-  struct type *vtype = VALUE_TYPE (varray);
+  struct type *vtype = check_typedef (VALUE_TYPE (varray));
   LONGEST length = unpack_long (TYPE_FIELD_TYPE (vtype, 0),
 				VALUE_CONTENTS (varray)
 				+ TYPE_FIELD_BITPOS (vtype, 0) / 8);
@@ -2186,3 +2332,15 @@ cast_into_complex (type, val)
   else
     error ("cannot cast non-number to complex");
 }
+
+void
+_initialize_valops ()
+{
+#if 0
+  add_show_from_set
+    (add_set_cmd ("abandon", class_support, var_boolean, (char *)&auto_abandon,
+		  "Set automatic abandonment of expressions upon failure.",
+		  &setlist),
+     &showlist);
+#endif
+}
