@@ -13,8 +13,8 @@ Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with the GNU C Library; see the file COPYING.LIB.  If
-not, write to the Free Software Foundation, Inc., 675 Mass Ave,
-Cambridge, MA 02139, USA.  */
+not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 #include "mmprivate.h"
 
@@ -38,3 +38,11 @@ mvalloc (md, size)
 
   return (mmemalign (md, pagesize, size));
 }
+
+
+PTR
+valloc (size)
+  size_t size;
+{
+  return mvalloc ((PTR) NULL, size);
+}
