@@ -1,5 +1,5 @@
 /* Print SPARC instructions.
-   Copyright 1989, 1991, 1992, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1989, 91-93, 1995, 1996 Free Software Foundation, Inc.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,13 +13,39 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "ansidecl.h"
 #include "opcode/sparc.h"
 #include "dis-asm.h"
+#include "libiberty.h"
 #include <string.h>
 
+/* Bitmask of v9 architectures.  */
+#define MASK_V9 ((1 << SPARC_OPCODE_ARCH_V9) \
+		 | (1 << SPARC_OPCODE_ARCH_V9A))
+/* 1 if INSN is for v9 only.  */
+#define V9_ONLY_P(insn) (! ((insn)->architecture & ~MASK_V9))
+/* 1 if INSN is for v9.  */
+#define V9_P(insn) (((insn)->architecture & MASK_V9) != 0)
+
+/* For faster lookup, after insns are sorted they are hashed.  */
+/* ??? I think there is room for even more improvement.  */
+
+#define HASH_SIZE 256
+/* It is important that we only look at insn code bits as that is how the
+   opcode table is hashed.  OPCODE_BITS is a table of valid bits for each
+   of the main types (0,1,2,3).  */
+static int opcode_bits[4] = { 0x01c00000, 0x0, 0x01f80000, 0x01f80000 };
+#define HASH_INSN(INSN) \
+  ((((INSN) >> 24) & 0xc0) | (((INSN) & opcode_bits[((INSN) >> 30) & 3]) >> 19))
+struct opcode_hash {
+  struct opcode_hash *next;
+  struct sparc_opcode *opcode;
+};
+static struct opcode_hash *opcode_hash_table[HASH_SIZE];
+static void build_hash_table ();
+
 /* Sign-extend a value which is N bits long.  */
 #define	SEX(value, bits) \
 	((((int)(value)) << ((8 * sizeof (int)) - bits))	\
@@ -34,19 +60,16 @@ static  char *reg_names[] =
   "f8", "f9", "f10", "f11", "f12", "f13", "f14", "f15",	
   "f16", "f17", "f18", "f19", "f20", "f21", "f22", "f23",
   "f24", "f25", "f26", "f27", "f28", "f29", "f30", "f31",
-#ifndef NO_V9
   "f32", "f33", "f34", "f35", "f36", "f37", "f38", "f39",	
   "f40", "f41", "f42", "f43", "f44", "f45", "f46", "f47",	
   "f48", "f49", "f50", "f51", "f52", "f53", "f54", "f55",
   "f56", "f57", "f58", "f59", "f60", "f61", "f62", "f63",
 /* psr, wim, tbr, fpsr, cpsr are v8 only.  */
-#endif
   "y", "psr", "wim", "tbr", "pc", "npc", "fpsr", "cpsr"
 };
 
 #define	freg_names	(&reg_names[4 * 8])
 
-#ifndef NO_V9
 /* These are ordered according to there register number in
    rdpr and wrpr insns.  */
 static char *v9_priv_reg_names[] =
@@ -56,87 +79,105 @@ static char *v9_priv_reg_names[] =
   "wstate", "fq"
   /* "ver" - special cased */
 };
-#endif
 
-/* FIXME--need to deal with byte order (probably using masking and
-   shifting rather than bitfields is easiest).  */
+/* Macros used to extract instruction fields.  Not all fields have
+   macros defined here, only those which are actually used.  */
 
-union sparc_insn
-  {
-    unsigned long int code;
-    struct
-      {
-	unsigned int anop:2;
-#define	op	ldst.anop
-	unsigned int anrd:5;
-#define	rd	ldst.anrd
-	unsigned int op3:6;
-	unsigned int anrs1:5;
-#define	rs1	ldst.anrs1
-	unsigned int i:1;
-	unsigned int anasi:8;
-#define	asi	ldst.anasi
-	unsigned int anrs2:5;
-#define	rs2	ldst.anrs2
-#define	shcnt	rs2
-      } ldst;
-    struct
-      {
-	unsigned int anop:2, anrd:5, op3:6, anrs1:5, i:1;
-	unsigned int IMM13:13;
-#define	imm13	IMM13.IMM13
-      } IMM13;
-    struct
-      {
-	unsigned int anop:2;
-	unsigned int a:1;
-	unsigned int cond:4;
-	unsigned int op2:3;
-	unsigned int DISP22:22;
-#define	disp22	branch.DISP22
-#define	imm22	disp22
-      } branch;
-#ifndef NO_V9
-    struct
-      {
-	unsigned int anop:2;
-	unsigned int a:1;
-	unsigned int z:1;
-	unsigned int rcond:3;
-	unsigned int op2:3;
-	unsigned int DISP16HI:2;
-	unsigned int p:1;
-	unsigned int _rs1:5;
-	unsigned int DISP16LO:14;
-#define	disp16(x) (((x).branch16.DISP16HI << 14) | (x).branch16.DISP16LO)
-      } branch16;
-#endif /* NO_V9 */
-    struct
-      {
-	unsigned int anop:2;
-	unsigned int adisp30:30;
-#define	disp30	call.adisp30
-      } call;
-  };
+#define X_RD(i) (((i) >> 25) & 0x1f)
+#define X_RS1(i) (((i) >> 14) & 0x1f)
+#define X_LDST_I(i) (((i) >> 13) & 1)
+#define X_ASI(i) (((i) >> 5) & 0xff)
+#define X_RS2(i) (((i) >> 0) & 0x1f)
+#define X_IMM13(i) (((i) >> 0) & 0x1fff)
+#define X_DISP22(i) (((i) >> 0) & 0x3fffff)
+#define X_IMM22(i) X_DISP22 (i)
+#define X_DISP30(i) (((i) >> 0) & 0x3fffffff)
+
+/* These are for v9.  */
+#define X_DISP16(i) (((((i) >> 20) & 3) << 14) | (((i) >> 0) & 0x3fff))
+#define X_DISP19(i) (((i) >> 0) & 0x7ffff)
+#define X_MEMBAR(i) ((i) & 0x7f)
+
+/* Here is the union which was used to extract instruction fields
+   before the shift and mask macros were written.
+
+   union sparc_insn
+     {
+       unsigned long int code;
+       struct
+	 {
+	   unsigned int anop:2;
+	   #define	op	ldst.anop
+	   unsigned int anrd:5;
+	   #define	rd	ldst.anrd
+	   unsigned int op3:6;
+	   unsigned int anrs1:5;
+	   #define	rs1	ldst.anrs1
+	   unsigned int i:1;
+	   unsigned int anasi:8;
+	   #define	asi	ldst.anasi
+	   unsigned int anrs2:5;
+	   #define	rs2	ldst.anrs2
+	   #define	shcnt	rs2
+	 } ldst;
+       struct
+	 {
+	   unsigned int anop:2, anrd:5, op3:6, anrs1:5, i:1;
+	   unsigned int IMM13:13;
+	   #define	imm13	IMM13.IMM13
+	 } IMM13;
+       struct
+	 {
+	   unsigned int anop:2;
+	   unsigned int a:1;
+	   unsigned int cond:4;
+	   unsigned int op2:3;
+	   unsigned int DISP22:22;
+	   #define	disp22	branch.DISP22
+	   #define	imm22	disp22
+	 } branch;
+       struct
+	 {
+	   unsigned int anop:2;
+	   unsigned int a:1;
+	   unsigned int z:1;
+	   unsigned int rcond:3;
+	   unsigned int op2:3;
+	   unsigned int DISP16HI:2;
+	   unsigned int p:1;
+	   unsigned int _rs1:5;
+	   unsigned int DISP16LO:14;
+	 } branch16;
+       struct
+	 {
+	   unsigned int anop:2;
+	   unsigned int adisp30:30;
+	   #define	disp30	call.adisp30
+	 } call;
+     };
+
+   */
 
 /* Nonzero if INSN is the opcode for a delayed branch.  */
 static int
 is_delayed_branch (insn)
-     union sparc_insn insn;
+     unsigned long insn;
 {
-  unsigned int i;
+  struct opcode_hash *op;
 
-  for (i = 0; i < NUMOPCODES; ++i)
+  for (op = opcode_hash_table[HASH_INSN (insn)]; op; op = op->next)
     {
-      CONST struct sparc_opcode *opcode = &sparc_opcodes[i];
-      if ((opcode->match & insn.code) == opcode->match
-	  && (opcode->lose & insn.code) == 0)
+      CONST struct sparc_opcode *opcode = op->opcode;
+      if ((opcode->match & insn) == opcode->match
+	  && (opcode->lose & insn) == 0)
 	return (opcode->flags & F_DELAYED);
     }
   return 0;
 }
 
-static int opcodes_sorted = 0;
+/* Nonzero of opcode table has been initialized.  */
+static int opcodes_initialized = 0;
+
 /* extern void qsort (); */
 static int compare_opcodes ();
 
@@ -147,26 +188,30 @@ static int compare_opcodes ();
    is preceded by a findable `sethi' and it either adds an immediate
    displacement to that register, or it is an `add' or `or' instruction
    on that register.  */
+
 int
 print_insn_sparc (memaddr, info)
      bfd_vma memaddr;
      disassemble_info *info;
 {
   FILE *stream = info->stream;
-  union sparc_insn insn;
-
+  bfd_byte buffer[4];
+  unsigned long insn;
   register unsigned int i;
+  register struct opcode_hash *op;
+  int sparc_v9_p = bfd_mach_sparc_v9_p (info->mach);
 
-  if (!opcodes_sorted)
+  if (!opcodes_initialized)
     {
-      qsort ((char *) sparc_opcodes, NUMOPCODES,
+      qsort ((char *) sparc_opcodes, sparc_num_opcodes,
 	     sizeof (sparc_opcodes[0]), compare_opcodes);
-      opcodes_sorted = 1;
+      build_hash_table (sparc_opcodes, opcode_hash_table, sparc_num_opcodes);
+      opcodes_initialized = 1;
     }
 
   {
     int status =
-      (*info->read_memory_func) (memaddr, (bfd_byte *) &insn, sizeof (insn), info);
+      (*info->read_memory_func) (memaddr, buffer, sizeof (buffer), info);
     if (status != 0)
       {
 	(*info->memory_error_func) (status, memaddr, info);
@@ -174,16 +219,31 @@ print_insn_sparc (memaddr, info)
       }
   }
 
+  insn = bfd_getb32 (buffer);
+
   info->insn_info_valid = 1;			/* We do return this info */
   info->insn_type = dis_nonbranch;		/* Assume non branch insn */
   info->branch_delay_insns = 0;			/* Assume no delay */
   info->target = 0;				/* Assume no target known */
 
-  for (i = 0; i < NUMOPCODES; ++i)
+  for (op = opcode_hash_table[HASH_INSN (insn)]; op; op = op->next)
     {
-      CONST struct sparc_opcode *opcode = &sparc_opcodes[i];
-      if ((opcode->match & insn.code) == opcode->match
-	  && (opcode->lose & insn.code) == 0)
+      CONST struct sparc_opcode *opcode = op->opcode;
+
+      /* ??? These architecture tests need to be more selective.  */
+
+      /* If the current architecture isn't sparc64, skip sparc64 insns.  */
+      if (!sparc_v9_p
+	  && V9_ONLY_P (opcode))
+	continue;
+
+      /* If the current architecture is sparc64, skip sparc32 only insns.  */
+      if (sparc_v9_p
+	  && ! V9_P (opcode))
+	continue;
+
+      if ((opcode->match & insn) == opcode->match
+	  && (opcode->lose & insn) == 0)
 	{
 	  /* Nonzero means that we have found an instruction which has
 	     the effect of adding or or'ing the imm13 field to rs1.  */
@@ -200,13 +260,17 @@ print_insn_sparc (memaddr, info)
 	     as rsd, and which has the i bit set?  */
 	  if ((opcode->match == 0x80102000 || opcode->match == 0x80002000)
 	  /*			  (or)				 (add)  */
-	      && insn.rs1 == insn.rd)
+	      && X_RS1 (insn) == X_RD (insn))
 	    imm_added_to_rs1 = 1;
 
-	  if (insn.rs1 != insn.rd
+	  if (X_RS1 (insn) != X_RD (insn)
 	      && strchr (opcode->args, 'r') != 0)
 	      /* Can't do simple format if source and dest are different.  */
 	      continue;
+	  if (X_RS2 (insn) != X_RD (insn)
+	      && strchr (opcode->args, 'O') != 0)
+	      /* Can't do simple format if source and dest are different.  */
+	      continue;
 
 	  (*info->fprintf_func) (stream, opcode->name);
 
@@ -227,7 +291,6 @@ print_insn_sparc (memaddr, info)
 		      is_annulled = 1;
 		      ++s;
 		      continue;
-#ifndef NO_V9
 		    case 'N':
 		      (*info->fprintf_func) (stream, "pn");
 		      ++s;
@@ -237,7 +300,6 @@ print_insn_sparc (memaddr, info)
 		      (*info->fprintf_func) (stream, "pt");
 		      ++s;
 		      continue;
-#endif	/* NO_V9 */
 
 		    default:
 		      break;
@@ -263,70 +325,70 @@ print_insn_sparc (memaddr, info)
 #define	reg(n)	(*info->fprintf_func) (stream, "%%%s", reg_names[n])
 		  case '1':
 		  case 'r':
-		    reg (insn.rs1);
+		    reg (X_RS1 (insn));
 		    break;
 
 		  case '2':
-		    reg (insn.rs2);
+		  case 'O':
+		    reg (X_RS2 (insn));
 		    break;
 
 		  case 'd':
-		    reg (insn.rd);
+		    reg (X_RD (insn));
 		    break;
 #undef	reg
 
 #define	freg(n)		(*info->fprintf_func) (stream, "%%%s", freg_names[n])
 #define	fregx(n)	(*info->fprintf_func) (stream, "%%%s", freg_names[((n) & ~1) | (((n) & 1) << 5)])
 		  case 'e':
-		    freg (insn.rs1);
+		    freg (X_RS1 (insn));
 		    break;
 		  case 'v':	/* double/even */
 		  case 'V':	/* quad/multiple of 4 */
-		    fregx (insn.rs1);
+		    fregx (X_RS1 (insn));
 		    break;
 
 		  case 'f':
-		    freg (insn.rs2);
+		    freg (X_RS2 (insn));
 		    break;
 		  case 'B':	/* double/even */
 		  case 'R':	/* quad/multiple of 4 */
-		    fregx (insn.rs2);
+		    fregx (X_RS2 (insn));
 		    break;
 
 		  case 'g':
-		    freg (insn.rd);
+		    freg (X_RD (insn));
 		    break;
 		  case 'H':	/* double/even */
 		  case 'J':	/* quad/multiple of 4 */
-		    fregx (insn.rd);
+		    fregx (X_RD (insn));
 		    break;
 #undef	freg
 #undef	fregx
 
 #define	creg(n)	(*info->fprintf_func) (stream, "%%c%u", (unsigned int) (n))
 		  case 'b':
-		    creg (insn.rs1);
+		    creg (X_RS1 (insn));
 		    break;
 
 		  case 'c':
-		    creg (insn.rs2);
+		    creg (X_RS2 (insn));
 		    break;
 
 		  case 'D':
-		    creg (insn.rd);
+		    creg (X_RD (insn));
 		    break;
 #undef	creg
 
 		  case 'h':
 		    (*info->fprintf_func) (stream, "%%hi(%#x)",
-					   0xFFFFFFFF & (int) insn.imm22 << 10);
+					   (0xFFFFFFFF
+					    & ((int) X_IMM22 (insn) << 10)));
 		    break;
 
 		  case 'i':
 		    {
-		      /* We cannot trust the compiler to sign-extend
-			 when extracting the bitfield, hence the shifts.  */
-		      int imm = SEX (insn.imm13, 13);
+		      int imm = SEX (X_IMM13 (insn), 13);
 
 		      /* Check to see whether we have a 1+i, and take
 			 note of that fact.
@@ -345,18 +407,15 @@ print_insn_sparc (memaddr, info)
 		    }
 		    break;
 
-#ifndef NO_V9
 		  case 'I':	/* 11 bit immediate.  */
 		  case 'j':	/* 10 bit immediate.  */
 		    {
-		      /* We cannot trust the compiler to sign-extend
-			 when extracting the bitfield, hence the shifts.  */
 		      int imm;
 
 		      if (*s == 'I')
-			imm = SEX (insn.imm13, 11);
+			imm = SEX (X_IMM13 (insn), 11);
 		      else
-			imm = SEX (insn.imm13, 10);
+			imm = SEX (X_IMM13 (insn), 10);
 
 		      /* Check to see whether we have a 1+i, and take
 			 note of that fact.
@@ -375,13 +434,37 @@ print_insn_sparc (memaddr, info)
 		    }
 		    break;
 
+		  case 'K':
+		    {
+		      int mask = X_MEMBAR (insn);
+		      int bit = 0x40, printed_one = 0;
+		      char *name;
+
+		      if (mask == 0)
+			(info->fprintf_func) (stream, "0");
+		      else
+			while (bit)
+			  {
+			    if (mask & bit)
+			      {
+				if (printed_one)
+				  (info->fprintf_func) (stream, "|");
+				name = sparc_decode_membar (bit);
+				(info->fprintf_func) (stream, "%s", name);
+				printed_one = 1;
+			      }
+			    bit >>= 1;
+			  }
+		      break;
+		    }
+
 		  case 'k':
-		    info->target = memaddr + (SEX (disp16 (insn), 16)) * 4;
+		    info->target = memaddr + SEX (X_DISP16 (insn), 16) * 4;
 		    (*info->print_address_func) (info->target, info);
 		    break;
 
 		  case 'G':
-		    info->target = memaddr + (SEX (insn.disp22, 19)) * 4;
+		    info->target = memaddr + SEX (X_DISP19 (insn), 19) * 4;
 		    (*info->print_address_func) (info->target, info);
 		    break;
 
@@ -421,49 +504,67 @@ print_insn_sparc (memaddr, info)
 		    break;
 
 		  case '?':
-		    if (insn.rs1 == 31)
+		    if (X_RS1 (insn) == 31)
 		      (*info->fprintf_func) (stream, "%%ver");
-		    else if ((unsigned) insn.rs1 < 16)
-		      (*info->fprintf_func) (stream, "%%%s", v9_priv_reg_names[insn.rs1]);
+		    else if ((unsigned) X_RS1 (insn) < 16)
+		      (*info->fprintf_func) (stream, "%%%s",
+					     v9_priv_reg_names[X_RS1 (insn)]);
 		    else
 		      (*info->fprintf_func) (stream, "%%reserved");
 		    break;
 
 		  case '!':
-		    if ((unsigned) insn.rd < 15)
-		      (*info->fprintf_func) (stream, "%%%s", v9_priv_reg_names[insn.rd]);
+		    if ((unsigned) X_RD (insn) < 15)
+		      (*info->fprintf_func) (stream, "%%%s",
+					     v9_priv_reg_names[X_RD (insn)]);
 		    else
 		      (*info->fprintf_func) (stream, "%%reserved");
 		    break;
-		    break;
-#endif	/* NO_V9 */
+
+		  case '*':
+		    {
+		      char *name = sparc_decode_prefetch (X_RD (insn));
+
+		      if (name)
+			(*info->fprintf_func) (stream, "%s", name);
+		      else
+			(*info->fprintf_func) (stream, "%d", X_RD (insn));
+		      break;
+		    }
 
 		  case 'M':
-		    (*info->fprintf_func) (stream, "%%asr%d", insn.rs1);
+		    (*info->fprintf_func) (stream, "%%asr%d", X_RS1 (insn));
 		    break;
 		    
 		  case 'm':
-		    (*info->fprintf_func) (stream, "%%asr%d", insn.rd);
+		    (*info->fprintf_func) (stream, "%%asr%d", X_RD (insn));
 		    break;
 		    
 		  case 'L':
-		    info->target = memaddr + insn.disp30 * 4;
+		    info->target = memaddr + SEX (X_DISP30 (insn), 30) * 4;
 		    (*info->print_address_func) (info->target, info);
 		    break;
 
 		  case 'n':
 		    (*info->fprintf_func)
-		      (stream, "%#x", (SEX (insn.disp22, 22)));
+		      (stream, "%#x", SEX (X_DISP22 (insn), 22));
 		    break;
 
 		  case 'l':
-		    info->target = memaddr + (SEX (insn.disp22, 22)) * 4;
+		    info->target = memaddr + SEX (X_DISP22 (insn), 22) * 4;
 		    (*info->print_address_func) (info->target, info);
 		    break;
 
 		  case 'A':
-		    (*info->fprintf_func) (stream, "(%d)", (int) insn.asi);
-		    break;
+		    {
+		      char *name = sparc_decode_asi (X_ASI (insn));
+
+		      if (name)
+			(*info->fprintf_func) (stream, "%s", name);
+		      else
+			(*info->fprintf_func) (stream, "(%d)", X_ASI (insn));
+		      break;
+		    }
 
 		  case 'C':
 		    (*info->fprintf_func) (stream, "%%csr");
@@ -493,9 +594,28 @@ print_insn_sparc (memaddr, info)
 		    (*info->fprintf_func) (stream, "%%wim");
 		    break;
 
+		  case 'x':
+		    (*info->fprintf_func) (stream, "%d",
+					   ((X_LDST_I (insn) << 8)
+					    + X_ASI (insn)));
+		    break;
+
 		  case 'y':
 		    (*info->fprintf_func) (stream, "%%y");
 		    break;
+
+		  case 'u':
+		  case 'U':
+		    {
+		      int val = *s == 'U' ? X_RS1 (insn) : X_RD (insn);
+		      char *name = sparc_decode_sparclet_cpreg (val);
+
+		      if (name)
+			(*info->fprintf_func) (stream, "%s", name);
+		      else
+			(*info->fprintf_func) (stream, "%%cpreg(%d)", val);
+		      break;
+		    }
 		  }
 	      }
 	  }
@@ -508,13 +628,13 @@ print_insn_sparc (memaddr, info)
 	     and its symbolic value.  */
 	  if (imm_added_to_rs1)
 	    {
-	      union sparc_insn prev_insn;
+	      unsigned long prev_insn;
 	      int errcode;
 
 	      errcode =
 		(*info->read_memory_func)
-		  (memaddr - 4,
-		   (bfd_byte *)&prev_insn, sizeof (prev_insn), info);
+		  (memaddr - 4, buffer, sizeof (buffer), info);
+	      prev_insn = bfd_getb32 (buffer);
 
 	      if (errcode == 0)
 		{
@@ -528,9 +648,11 @@ print_insn_sparc (memaddr, info)
 		     */
 
 		  if (is_delayed_branch (prev_insn))
-		    errcode = (*info->read_memory_func)
-		      (memaddr - 8, (bfd_byte *)&prev_insn, sizeof (prev_insn),
-		       info);
+		    {
+		      errcode = (*info->read_memory_func)
+			(memaddr - 8, buffer, sizeof (buffer), info);
+		      prev_insn = bfd_getb32 (buffer);
+		    }
 		}
 
 	      /* If there was a problem reading memory, then assume
@@ -538,13 +660,13 @@ print_insn_sparc (memaddr, info)
 	      if (errcode == 0)
 		{
 		  /* Is it sethi to the same register?  */
-		  if ((prev_insn.code & 0xc1c00000) == 0x01000000
-		      && prev_insn.rd == insn.rs1)
+		  if ((prev_insn & 0xc1c00000) == 0x01000000
+		      && X_RD (prev_insn) == X_RS1 (insn))
 		    {
 		      (*info->fprintf_func) (stream, "\t! ");
 		      info->target = 
-			(0xFFFFFFFF & (int) prev_insn.imm22 << 10)
-			| SEX (insn.imm13, 13);
+			(0xFFFFFFFF & (int) X_IMM22 (prev_insn) << 10)
+			| SEX (X_IMM13 (insn), 13);
 		      (*info->print_address_func) (info->target, info);
 		      info->insn_type = dis_dref;
 		      info->data_size = 4;  /* FIXME!!! */
@@ -565,13 +687,13 @@ print_insn_sparc (memaddr, info)
 		info->branch_delay_insns = 1;
 	    }
 
-	  return sizeof (insn);
+	  return sizeof (buffer);
 	}
     }
 
   info->insn_type = dis_noninsn;	/* Mark as non-valid instruction */
-  (*info->fprintf_func) (stream, "%#8x", insn.code);
-  return sizeof (insn);
+  (*info->fprintf_func) (stream, "unknown");
+  return sizeof (buffer);
 }
 
 /* Compare opcodes A and B.  */
@@ -626,6 +748,10 @@ compare_opcodes (a, b)
 	return x1 - x0;
     }
 
+  /* Put non-sparc64 insns ahead of sparc64 ones.  */
+  if (V9_ONLY_P (op0) != V9_ONLY_P (op1))
+    return V9_ONLY_P (op0) - V9_ONLY_P (op1);
+
   /* They are functionally equal.  So as long as the opcode table is
      valid, we can put whichever one first we want, on aesthetic grounds.  */
 
@@ -675,9 +801,68 @@ compare_opcodes (a, b)
       }
   }
 
+  /* Put 1,i before i,1.  */
+  {
+    int i0 = strncmp (op0->args, "i,1", 3) == 0;
+    int i1 = strncmp (op1->args, "i,1", 3) == 0;
+
+    if (i0 ^ i1)
+      return i0 - i1;
+  }
+
   /* They are, as far as we can tell, identical.
      Since qsort may have rearranged the table partially, there is
      no way to tell which one was first in the opcode table as
      written, so just say there are equal.  */
   return 0;
 }
+
+/* Build a hash table from the opcode table.  */
+
+static void
+build_hash_table (table, hash_table, num_opcodes)
+     struct sparc_opcode *table;
+     struct opcode_hash **hash_table;
+     int num_opcodes;
+{
+  register int i;
+  int hash_count[HASH_SIZE];
+  static struct opcode_hash *hash_buf = NULL;
+
+  /* Start at the end of the table and work backwards so that each
+     chain is sorted.  */
+
+  memset (hash_table, 0, HASH_SIZE * sizeof (hash_table[0]));
+  memset (hash_count, 0, HASH_SIZE * sizeof (hash_count[0]));
+  if (hash_buf != NULL)
+    free (hash_buf);
+  hash_buf = (struct opcode_hash *) xmalloc (sizeof (struct opcode_hash) * num_opcodes);
+  for (i = num_opcodes - 1; i >= 0; --i)
+    {
+      register int hash = HASH_INSN (sparc_opcodes[i].match);
+      register struct opcode_hash *h = &hash_buf[i];
+      h->next = hash_table[hash];
+      h->opcode = &sparc_opcodes[i];
+      hash_table[hash] = h;
+      ++hash_count[hash];
+    }
+
+#if 0 /* for debugging */
+  {
+    int min_count = num_opcodes, max_count = 0;
+    int total;
+
+    for (i = 0; i < HASH_SIZE; ++i)
+      {
+        if (hash_count[i] < min_count)
+	  min_count = hash_count[i];
+	if (hash_count[i] > max_count)
+	  max_count = hash_count[i];
+	total += hash_count[i];
+      }
+
+    printf ("Opcode hash table stats: min %d, max %d, ave %f\n",
+	    min_count, max_count, (double) total / HASH_SIZE);
+  }
+#endif
+}
